def destroy_spanners_attached_to_component(component, klass=None):
    r'''.. versionadded:: 1.1

    Destroy spanners of `klass` attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beam = beamtools.BeamSpanner(staff.leaves)
        >>> slur = spannertools.SlurSpanner(staff.leaves)
        >>> trill = spannertools.TrillSpanner(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ( \startTrillSpan
            d'8
            e'8
            f'8 ] ) \stopTrillSpan
        }

    ::

        >>> spanners = spannertools.destroy_spanners_attached_to_component(staff[0])

    ::

        >>> f(staff)
        \new Staff {
            c'8 \startTrillSpan
            d'8
            e'8
            f'8 \stopTrillSpan
        }

    Destroy all spanners when `klass` is none.

    Return tuple of zero or more empty spanners.

    Order of spanners in return value can not be predicted.
    '''
    from abjad.tools import spannertools

    result = []
    for spanner in spannertools.get_spanners_attached_to_component(component):
        if klass is None or isinstance(spanner, klass):
            spanner.clear()
            result.append(spanner)

    return tuple(result)



from abjad.tools import componenttools


def get_tie_chains_masked_by_components(components):
    r'''Get tie chains masked by `components`:

    ::

        >>> staff = Staff("abj: | 2/4 c'4 d'4 ~ || 2/4 d'4 e'4 ~ || 2/4 e'4 f'4 |")
        >>> f(staff)
        \new Staff {
            {
                \time 2/4
                c'4
                d'4 ~
            }
            {
                d'4
                e'4 ~
            }
            {
                e'4
                f'4
            }
        }

    ::

        >>> for tie_chain in tietools.get_tie_chains_masked_by_components(staff.leaves):
        ...     tie_chain
        ... 
        TieChain(Note("c'4"),)
        TieChain(Note("d'4"), Note("d'4"))
        TieChain(Note("e'4"), Note("e'4"))
        TieChain(Note("f'4"),)

    ::

        >>> for tie_chain in tietools.get_tie_chains_masked_by_components(staff[1].leaves):
        ...     tie_chain
        ... 
        TieChain(Note("d'4"),)
        TieChain(Note("e'4"),)

    ::

        >>> for tie_chain in tietools.get_tie_chains_masked_by_components(staff[1:]):
        ...     tie_chain
        ... 
        TieChain(Note("d'4"),)
        TieChain(Note("e'4"), Note("e'4"))
        TieChain(Note("f'4"),)

    Return list of zero or more (possibly masked) tie chains.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import tietools

    assert componenttools.all_are_components(components)

    leaves_in_components = set(iterationtools.iterate_leaves_in_expr(components))

    tie_chains = []
    for leaf in leaves_in_components:
        tie_chain = tietools.get_tie_chain(leaf)
        if tie_chain not in tie_chains:
            tie_chains.append(tie_chain)

    result = []
    for tie_chain in tie_chains:
        leaves_in_tie_chain = set(iterationtools.iterate_leaves_in_expr(tie_chain))
        leaves_intersecting = leaves_in_components.intersection(leaves_in_tie_chain)
        result.append(tietools.TieChain(tuple(sorted(leaves_intersecting,
            key=lambda x: x.start_offset))))
    result.sort(key=lambda x: x[0].start_offset)

    return result




def get_context_mark_attached_to_component(component, klasses=None):
    r'''.. versionadded:: 2.3

    Get context mark attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.ClefMark('treble')(staff)
        ClefMark('treble')(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \clef "treble"
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> contexttools.get_context_mark_attached_to_component(staff)
        ClefMark('treble')(Staff{4})

    Return context mark.

    Raise missing mark error when no context mark attaches to `component`.
    '''
    from abjad.tools import contexttools

    if klasses is None:
        klasses = (contexttools.ContextMark,)

    context_marks = contexttools.get_context_marks_attached_to_component(component, klasses=klasses)

    if len(context_marks) == 0:
        raise MissingMarkError
    elif 1 < len(context_marks):
        raise ExtraMarkError
    else:
        return context_marks[0]



def get_lineage_of_component_that_stop_with_component(component):
    r'''.. versionadded:: 2.9

    Get lineage of `component` that stop with `component`::

        >>> staff = Staff(r"c' << \new Voice { d'8 e'8 } \new Voice { d''8 e''8 } >> f'4")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            <<
                \new Voice {
                    d'8
                    e'8
                }
                \new Voice {
                    d''8
                    e''8
                }
            >>
            f'4
        }

    ::

        >>> staff[1][0]
        Voice{2}

    ::

        >>> componenttools.get_lineage_of_component_that_stop_with_component(staff[1][0])
        [<<Voice{2}, Voice{2}>>, Voice{2}, Note("e'8")]

    Return list of all components in the lineage of `component` that
    stop with `component`.

    The list always includes `component`.
    '''
    from abjad.tools import componenttools

    # initialize result
    result = []
    
    # add parentage of component that start with component
    result.extend(componenttools.get_improper_parentage_of_component_that_stop_with_component(component))

    # remove component
    result.remove(component)

    # component and descendents of component that start with component
    result.extend(componenttools.get_improper_descendents_of_component_that_stop_with_component(component))

    # return result
    return result



def label_tie_chains_in_expr_with_tie_chain_duration(expr, markup_direction=Down):
    r'''Label tie chains in `expr` with prolated tie chain duration::

        >>> staff = Staff(r"\times 2/3 { c'8 ~ c'8 c'8 ~ } c'8")
        >>> labeltools.label_tie_chains_in_expr_with_tie_chain_duration(staff)

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8 ~
                    _ \markup {
                        \small
                            1/6
                        }
                c'8
                c'8 ~
                    _ \markup {
                        \small
                            5/24
                        }
            }
            c'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.
    '''
    from abjad.tools.labeltools._label_leaves_in_expr_with_leaf_durations import \
        _label_leaves_in_expr_with_leaf_durations

    show = ['prolated']
    return _label_leaves_in_expr_with_leaf_durations(
        expr, markup_direction=markup_direction, show=show, ties='together')



import abc
from abjad.tools import stringtools
from abjad.tools.marktools.Mark import Mark


class DirectedMark(Mark):
    '''Abstract base class of Marks which possess a vertical, typographic direction, 
    i.e. above or below the staff.
    '''    

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_direction', '_format_slot')

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, *args, **kwargs):
        Mark.__init__(self, *args)
        if 'direction' in kwargs:
            self.direction = kwargs['direction']
        else:
            self.direction = None
    
    ### PUBLIC PROPERTIES ###

    @apply
    def direction():
        def fget(self):
            return self._direction
        def fset(self, arg):
            self._direction = stringtools.arg_to_tridirectional_ordinal_constant(arg)
        return property(**locals())



from abjad.tools import documentationtools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript
import argparse
import os


class AbjDevScript(DeveloperScript):
    '''`AbjDevScript` is the commandline entry-point to the Abjad developer scripts catalog.

    Can be accessed on the commandline via `abj-dev` or `ajv`:

    ::

        bash$ abj-dev
        usage: abj-dev [-h] [--version]
                       
                       {help,list,api,book,clean,count,doctest,grep,new,rename,replace,svn}
                       ...

        Entry-point to Abjad developer scripts catalog.

        optional arguments:
          -h, --help            show this help message and exit
          --version             show program's version number and exit

        subcommands:
          {help,list,api,book,clean,count,doctest,grep,new,rename,replace,svn}
            help                print subcommand help
            list                list subcommands
            api                 Build the Abjad APIs.
            book                Preprocess HTML, LaTeX or ReST source with Abjad.
            clean               Clean .pyc, __pycache__ and tmp* files and folders
                                from PATH.
            count               "count"-related subcommands
            doctest             Run doctests on all modules in current path.
            grep                grep PATTERN in PATH
            new                 "new"-related subcommands
            rename              Rename public modules.
            replace             "replace"-related subcommands
            svn                 "svn"-related subcommands

    `ajv` supports subcommands similar to `svn`:

    ::

        bash$ ajv count -h
        usage: abj-dev count [-h] {linewidths,tools} ...

        optional arguments:
          -h, --help          show this help message and exit

        count subcommands:
          {linewidths,tools}
            linewidths        Count maximum line-width of all modules in PATH.
            tools             Count tools in PATH.

    Return `AbjDevScript` instance.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, args=None):
        if args is None:
            args = self.argument_parser.parse_known_args()
        else:
            if isinstance(args, str):
                args = args.split()
            elif not isinstance(args, (list, tuple)):
                raise ValueError
            args = self.argument_parser.parse_known_args(args)
        self.process_args(args)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def developer_script_aliases(self):
        scripting_groups = []
        aliases = {}
        for klass in self.developer_script_classes:
            instance = klass()

            if instance.alias is not None:

                if instance.scripting_group is not None:
                    scripting_groups.append(instance.scripting_group)
                    entry = (instance.scripting_group, instance.alias)
                    if (instance.scripting_group,) in aliases:
                        raise Exception('Alias conflict between scripting group {!r} and {}'.format(
                            instance.scripting_group, aliases[(instance.scripting_group,)].__name__))
                    if entry in aliases:
                        raise Exception('Alias conflict between {} and {}'.format(
                            aliases[entry].__name__ and klass.__name__))
                    aliases[entry] = klass

                else:
                    entry = (instance.alias,)
                    if entry in scripting_groups:
                        raise Exception('Alias conflict between {} and scripting group {!r}'.format(
                            klass.__name__, instance.alias))
                    if entry in aliases:
                        raise Exception('Alias conflict be {} and {}'.format(
                            klass.__name__, aliases[entry]))
                    aliases[(instance.alias,)] = klass

            else:
                if instance.program_name in scripting_groups:
                    raise Exception('Alias conflict between {} and scripting group {!r}'.format(
                        klass.__name__, instance.program_name))
                aliases[(instance.program_name,)] = klass

        aliasdict = {}

        for key, value in aliases.iteritems():
            if len(key) == 1:
                aliasdict[key[0]] = value
            else:
                if key[0] not in aliasdict:
                    aliasdict[key[0]] = { }
                aliasdict[key[0]][key[1]] = value

        return aliasdict

    @property
    def developer_script_classes(self):
        from abjad.tools.developerscripttools.get_developer_script_classes import get_developer_script_classes
        klasses = get_developer_script_classes()
        klasses.remove(self.__class__)
        return klasses

    @property
    def developer_script_program_names(self):
        program_names = {}
        for klass in self.developer_script_classes:
            instance = klass()
            program_names[instance.program_name] = klass
        return program_names

    @property
    def long_description(self):
        return None

    @property
    def short_description(self):
        return 'Entry-point to Abjad developer scripts catalog.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        args, unknown_args = args

        if args.subparser_name == 'help':
            aliases = self.developer_script_aliases
            program_names = self.developer_script_program_names
            klass = None
            if len(unknown_args) == 2 and \
                unknown_args[0] in aliases and \
                unknown_args[1] in aliases[unknown_args[0]]:
                klass = aliases[unknown_args[0]][unknown_args[1]]
            elif len(unknown_args) == 1 and \
                unknown_args[0] in aliases and \
                not isinstance(aliases[unknown_args[0]], dict):
                klass = aliases[unknown_args[0]]
            elif len(unknown_args) == 1 and \
                unknown_args[0] in program_names:
                klass = program_names[unknown_args[0]]

            if klass:
                instance = klass()
                print instance.formatted_help
            else:
                print 'Cannot resolve {} to subcommand.'.format(unknown_args)

        elif args.subparser_name == 'list':
            entries = []
            for klass in self.developer_script_classes:
                instance = klass()
                alias = ''
                if instance.alias is not None:
                    if instance.scripting_group is not None:
                        alias = '\n[{} {}]'.format(instance.scripting_group, instance.alias)
                    else:
                        alias = '\n[{}]'.format(instance.alias)
                entries.append('{}{}\n\t{}'.format(instance.program_name, alias, instance.short_description))
            print ''
            print '\n\n'.join(entries)
            print ''

        else:
            if hasattr(args, 'subsubparser_name'):
                klass = self.developer_script_aliases[args.subparser_name][args.subsubparser_name]
            else:
                klass = self.developer_script_aliases[args.subparser_name]
            instance = klass()
            instance(unknown_args)

    def setup_argument_parser(self, parser):

        subparsers = parser.add_subparsers(
            dest='subparser_name',
            title='subcommands',
            )
        
        info_subparser = subparsers.add_parser('help',
            add_help=False,
            help='print subcommand help'
            )

        list_subparser = subparsers.add_parser('list',
            add_help=False,
            help='list subcommands',
            )

        aliasdict = self.developer_script_aliases
        for key in sorted(aliasdict):
            if not isinstance(aliasdict[key], dict):
                klass = aliasdict[key]
                instance = klass()
                klass_subparser = subparsers.add_parser(key,
                    add_help=False,
                    help=instance.short_description,
                    )
            else:
                group_subparser = subparsers.add_parser(key,
                    help='"{}"-related subcommands'.format(key),
                    )
                group_subparsers = group_subparser.add_subparsers(
                    dest='subsubparser_name'.format(key),
                    title='{} subcommands'.format(key),
                    )
                for subkey in sorted(aliasdict[key]):
                    klass = aliasdict[key][subkey]
                    instance = klass()
                    group_subparsers.add_parser(subkey,
                        add_help=False,
                        help=instance.short_description
                        )



# TODO: Write tests #
def analyze_incomplete_tonal_function(expr, key_signature):
    '''.. versionadded:: 2.0

    Analyze tonal function of `expr` according to `key_signature`::

        >>> chord = Chord("<c' e'>4")
        >>> key_signature = contexttools.KeySignatureMark('g', 'major')
        >>> tonalitytools.analyze_incomplete_tonal_function(chord, key_signature)
        IVMajorTriadInRootPosition

    Return tonal function.
    '''
    from abjad.tools import tonalitytools

    if isinstance(expr, tonalitytools.ChordClass):
        chord_class = expr
    else:
        chord_class = tonalitytools.analyze_incomplete_chord(expr)
    root = chord_class.root
    scale = tonalitytools.Scale(key_signature)
    scale_degree = scale.named_chromatic_pitch_class_to_scale_degree(root)
    quality = chord_class.quality_indicator.quality_string
    extent = chord_class.extent
    inversion = chord_class.inversion
    tonal_function = tonalitytools.TonalFunction(scale_degree, quality, extent, inversion)

    return tonal_function



from abjad.tools.abctools import AbjadObject


class LilyPondFraction(AbjadObject):
    '''Model of a fraction in LilyPond.

    Not composer-safe.

    Used internally by LilyPondParser.
    '''

    __slots__ = ('numerator', 'denominator')

    def __init__(self, numerator, denominator):
        self.numerator = numerator
        self.denominator = denominator



from abjad import *


def test_LayoutBlock_lilypond_command_mark_01():

    staff = Staff("fs'2 fs'2 gs'2 gs'2")
    score = Score([staff])
    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    lilypond_file.layout_block.append(marktools.LilyPondCommandMark('accidentalStyle forget'))

    assert lilypond_file.layout_block.lilypond_format == '\\layout {\n\t\\accidentalStyle forget\n}'



from abjad.tools import durationtools


# TODO: implement related function to shorten from right edge.
# TODO: allow large values of `prolated_duration` to empty container contents.
def shorten_component_by_duration(component, duration):
    r'''.. versionadded:: 2.0

    Shorten `component` by prolated `duration`.

    Example 1. Shorten container and produce note with dotted duration::

        >>> staff = Staff("c'8 [ d'8 e'8 f'8 ]")

    ::

        >>> componenttools.shorten_component_by_duration(staff, Duration(1, 32))

    ::

        >>> f(staff)
        \new Staff {
            c'16. [
            d'8
            e'8
            f'8 ]
        }

    ::
    
        >>> show(staff) # doctest: +SKIP

    Example 2. Shorten container and produce note with tied duration::

        >>> staff = Staff("c'8 [ d'8 e'8 f'8 ]")

    ::

        >>> componenttools.shorten_component_by_duration(staff, Duration(3, 64))

    ::

        >>> f(staff)
        \new Staff {
            c'16 [ ~
            c'64
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Example 3. Shorten container and produce note with non-power-of-two duration::

        >>> staff = Staff("c'8 [ d'8 e'8 f'8 ]")

    ::

        >>> componenttools.shorten_component_by_duration(staff, Duration(1, 24))

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8 [
            }
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.
    '''
    from abjad.tools import componenttools
    from abjad.tools import leaftools

    assert isinstance(component, componenttools.Component)
    duration = durationtools.Duration(duration)

    if component.prolated_duration <= duration:
        raise Exception('component durations must be positive.')

    if isinstance(component, leaftools.Leaf):
        new_prolated_duration = component.prolated_duration - duration
        prolation = component.prolation
        new_written_duration = new_prolated_duration / prolation
        leaftools.set_preprolated_leaf_duration(component, new_written_duration)
    else:
        container = component
        components, accumulated_duration = \
            componenttools.get_leftmost_components_with_total_duration_at_most(container[:], duration)
        del(container[:len(components)])
        remaining_subtrahend_duration = duration - accumulated_duration
        componenttools.shorten_component_by_duration(container[0], remaining_subtrahend_duration)



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools.abctools import AbjadObject


class QTargetBeat(AbjadObject):
    '''Representation of a single "beat" in a quantization target:

    ::

        >>> beatspan = (1, 8)
        >>> offset_in_ms = 1500
        >>> search_tree = quantizationtools.UnweightedSearchTree({3: None})
        >>> tempo = contexttools.TempoMark((1, 4), 56)

    ::

        >>> q_target_beat = quantizationtools.QTargetBeat(
        ...     beatspan=beatspan,
        ...     offset_in_ms=offset_in_ms,
        ...     search_tree=search_tree,
        ...     tempo=tempo,
        ...     )

    ::

        >>> q_target_beat
        quantizationtools.QTargetBeat(
            beatspan=durationtools.Duration(1, 8),
            offset_in_ms=durationtools.Offset(1500, 1),
            search_tree=quantizationtools.UnweightedSearchTree(
                definition={   3: None}
                ),
            tempo=contexttools.TempoMark(
                durationtools.Duration(1, 4),
                56
                )
            )

    Not composer-safe.

    Used internally by quantizationtools.Quantizer.

    Return ``QTargetBeat`` instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_beatspan', '_distances', '_grouping', '_offset_in_ms',
        '_q_events', '_q_grid', '_q_grids', '_search_tree', '_tempo')

    ### INITIALIZER ###

    def __init__(self, beatspan=None, offset_in_ms=None, search_tree=None, tempo=None):
        from abjad.tools import quantizationtools

        beatspan = durationtools.Duration(beatspan)
        offset_in_ms = durationtools.Offset(offset_in_ms)

        if search_tree is None:
            search_tree = quantizationtools.UnweightedSearchTree()
        assert isinstance(search_tree, quantizationtools.SearchTree)
        tempo = contexttools.TempoMark(tempo)
        assert not tempo.is_imprecise

        q_events = []
        q_grids = []
        
        self._beatspan = beatspan
        self._distances = {}
        self._offset_in_ms = offset_in_ms
        self._q_events = q_events
        self._q_grid = None
        self._q_grids = q_grids
        self._search_tree = search_tree
        self._tempo = tempo

    ### SPECIAL METHODS ###

    def __call__(self, job_id):
        from abjad.tools import quantizationtools
        if not self.q_events:
            return None
        assert all([isinstance(x, quantizationtools.QEvent) for x in self.q_events])
        q_event_proxies = []
        for q_event in self.q_events:
            q_event_proxy = quantizationtools.QEventProxy(
                q_event, self.offset_in_ms, self.offset_in_ms + self.duration_in_ms)
            q_event_proxies.append(q_event_proxy)
        return quantizationtools.QuantizationJob(job_id, self.search_tree, q_event_proxies)

    def __repr__(self):
        return self._tools_package_qualified_indented_repr

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def beatspan(self):
        '''The beatspan of the ``QTargetBeat``:

        ::

            >>> q_target_beat.beatspan
            Duration(1, 8)

        Return Duration.
        '''
        return self._beatspan

    @property
    def distances(self):
        '''A list of computed distances between the ``QEventProxies``
        associated with a ``QTargetBeat`` instance, and each ``QGrid``
        generated for that beat.

        Used internally by the ``Quantizer``.

        Return tuple.
        '''
        return self._distances

    @property
    def duration_in_ms(self):
        '''The duration in milliseconds of the ``QTargetBeat``:

        ::

            >>> q_target_beat.duration_in_ms
            Duration(3750, 7)

        Return Duration instance.
        '''
        from abjad.tools import quantizationtools
        return quantizationtools.tempo_scaled_duration_to_milliseconds(
            self.beatspan, self.tempo)

    @property
    def offset_in_ms(self):
        '''The offset in milliseconds of the ``QTargetBeat``:

        ::

            >>> q_target_beat.offset_in_ms
            Offset(1500, 1)

        Return Offset instance.
        '''
        return self._offset_in_ms

    @property
    def q_events(self):
        '''A list for storing ``QEventProxy`` instances.

        Used internally by the ``Quantizer``.

        Return list.
        '''
        return self._q_events

    @property
    def q_grid(self):
        '''The ``QGrid`` instance selected by a ``Heuristic``.

        Used internally by the ``Quantizer``.

        Return ``QGrid`` instance.
        '''
        return self._q_grid
    
    @property
    def q_grids(self):
        '''A tuple of ``QGrids`` generated by a ``QuantizationJob``.

        Used internally by the ``Quantizer``.

        Return tuple.
        '''
        return self._q_grids

    @property
    def search_tree(self):
        '''The search tree of the ``QTargetBeat``:

        ::

            >>> q_target_beat.search_tree
            UnweightedSearchTree(
                definition={   3: None}
                )

        Return ``SearchTree`` instance.
        '''
        return self._search_tree

    @property
    def tempo(self):
        '''The tempo of the ``QTargetBeat``:

        ::

            >>> q_target_beat.tempo
            TempoMark(Duration(1, 4), 56)

        Return ``TempoMark`` instance.
        '''
        return self._tempo



def get_all_unique_bounds_in_intervals(intervals):
    '''Find all unique starting and ending boundaries in `intervals`.

    Return list of Offsets.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    values = []
    for interval in tree:
        if interval.start not in values:
            values.append(interval.start)
        if interval.stop not in values:
            values.append(interval.stop)
    return tuple(sorted(values))



from abjad.tools import skiptools


def iterate_skips_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10


    Iterate skips forward in `expr`::

        >>> staff = Staff("<e' g' c''>8 a'8 s8 <d' f' b'>8 s2")

    ::

        >>> f(staff)
        \new Staff {
            <e' g' c''>8
            a'8
            s8
            <d' f' b'>8
            s2
        }

    ::

        >>> for skip in iterationtools.iterate_skips_in_expr(staff):
        ...   skip
        Skip('s8')
        Skip('s2')

    Iterate skips backwards in `expr`:

    ::

        >>> for skip in iterationtools.iterate_skips_in_expr(staff, reverse=True):
        ...   skip
        Skip('s2')
        Skip('s8')

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=skiptools.Skip, reverse=reverse, start=start, stop=stop)



def get_spanner_format_contributions(component):
    '''Get spanner format contributions for `leaf` as a dictionary 
    of format_slot:contributions key:value pairs.

    Return dict.
    '''

    from abjad.tools import containertools
    from abjad.tools import spannertools

    result = {
        'after': [],
        'before': [],
        'closing': [],
        'opening': [],
        'right': [],
    }

    spanners = spannertools.get_spanners_attached_to_any_improper_parent_of_component(component)
    spanners = list(spanners)
    spanners.sort(lambda x, y: cmp(x.__class__.__name__, y.__class__.__name__))

    if isinstance(component, containertools.Container):
        before_contributions = result['before']
        after_contributions = result['after']
    else:
        before_contributions = result['opening']
        after_contributions = result['closing']
        
    stop_contributions = []
    other_contributions = []

    for spanner in spanners:

        # before
        if spanner._is_my_first_leaf(component):
            contributions = spanner.override._list_format_contributions('override', is_once = False)
            before_contributions.extend(contributions)
        if hasattr(spanner, '_format_before_leaf'):
            before_contributions.extend(spanner._format_before_leaf(component))
        else:
            before_contributions.extend(spanner._format._before(component))

        # after
        if hasattr(spanner, '_format_after_leaf'):
            after_contributions.extend(spanner._format_after_leaf(component))
        else:
            after_contributions.extend(spanner._format._after(component))
        if spanner._is_my_last_leaf(component):
            contributions = spanner.override._list_format_contributions('revert')
            after_contributions.extend(contributions)

        # right
        if hasattr(spanner, '_format_right_of_leaf'):
            contributions = spanner._format_right_of_leaf(component)
        else:
            contributions = spanner._format._right(component)
        if contributions:
            if spanner._is_my_last_leaf(component):
                stop_contributions.extend(contributions)
            else:
                other_contributions.extend(contributions)

    result['right'] = stop_contributions + other_contributions

    for key in result.keys():
        if not result[key]:
            del(result[key])

    return result




from abjad.tools import componenttools


def is_component_with_lilypond_command_mark_attached(expr, command_name=None):
    '''.. versionadded:: 2.0

    True when `expr` is component with LilyPond command mark attached::

        >>> note = Note("c'4")
        >>> marktools.LilyPondCommandMark('stemUp')(note)
        LilyPondCommandMark('stemUp')(c'4)

    ::

        >>> marktools.is_component_with_lilypond_command_mark_attached(note)
        True

    False otherwise::

        >>> note = Note("c'4")

    ::

        >>> marktools.is_component_with_lilypond_command_mark_attached(note)
        False

    Return boolean.
    '''
    from abjad.tools import marktools

    if isinstance(expr, componenttools.Component):
        for mark in marktools.get_lilypond_command_marks_attached_to_component(expr):
            if mark.command_name == command_name or command_name is None:
                return True

    return False



from abjad.tools import durationtools


def report_integer_tempo_rewrite_pairs(integer_tempo,
    maximum_numerator=None, maximum_denominator=None):
    '''.. versionadded:: 2.0

    Report `integer_tempo` rewrite pairs.

    Allow no tempo less than half `integer_tempo` or 
    greater than double `integer_tempo`::

        >>> tempotools.report_integer_tempo_rewrite_pairs(
        ...   58, maximum_numerator=8, maximum_denominator=8)
        2:1     29
        1:1     58
        2:3     87
        1:2     116

    With more lenient numerator and denominator::

        >>> tempotools.report_integer_tempo_rewrite_pairs(
        ...     58, maximum_numerator=30, maximum_denominator=30)
        2:1     29
        29:15   30
        29:16   32
        29:17   34
        29:18   36
        29:19   38
        29:20   40
        29:21   42
        29:22   44
        29:23   46
        29:24   48
        29:25   50
        29:26   52
        29:27   54
        29:28   56
        1:1     58
        29:30   60
        2:3     87
        1:2     116

    Return none.
    '''
    from abjad.tools import tempotools

    pairs = tempotools.rewrite_integer_tempo(
      integer_tempo, maximum_numerator=maximum_numerator, maximum_denominator=maximum_denominator)

    for multiplier, tempo in pairs:
      print '{}\t{}'.format(multiplier.prolation_string, tempo)



#def _get_spanner_format_contributions_for_leaf_slot(leaf, slot):
#    '''.. versionadded:: 2.0
#    '''
#    from abjad.tools import spannertools
#
#    result = []
#    spanners = spannertools.get_spanners_attached_to_any_improper_parent_of_component(leaf)
#    spanners = list(spanners)
#    spanners.sort(lambda x, y: cmp(x.__class__.__name__, y.__class__.__name__))
#    if slot == 'before':
#        for spanner in spanners:
#            spanner_contributions = []
#            if spanner._is_my_first_leaf(leaf):
#                contributions = spanner.override._list_format_contributions('override', is_once = False)
#                spanner_contributions.extend(contributions)
#            spanner_contributions.extend(spanner._format._before(leaf))
#            result.extend(spanner_contributions)
#    elif slot == 'after':
#        for spanner in spanners:
#            spanner_contributions = []
#            spanner_contributions.extend(spanner._format._after(leaf))
#            if spanner._is_my_last_leaf(leaf):
#                contributions = spanner.override._list_format_contributions('revert')
#                spanner_contributions.extend(contributions)
#            result.extend(spanner_contributions)
#    elif slot == 'right':
#        stop_contributions, other_contributions = [], []
#        for spanner in spanners:
#            contributions = spanner._format._right(leaf)
#            if contributions:
#                if spanner._is_my_last_leaf(leaf):
#                    stop_contributions.extend(contributions)
#                else:
#                    other_contributions.extend(contributions)
#        result = stop_contributions + other_contributions
#    return ['spanners', result]



from abjad.tools import abctools
from abjad.tools import schemetools
from abjad.tools import stringtools
from abjad.tools.marktools.DirectedMark.DirectedMark import DirectedMark


class Markup(DirectedMark):
    r'''Abjad model of LilyPond markup.

    Initialize from string:

    ::

        >>> markup = markuptools.Markup(r'\bold { "This is markup text." }')

    ::

        >>> markup
        Markup((MarkupCommand('bold', ['This is markup text.']),))

    ::

        >>> f(markup)
        \markup { \bold { "This is markup text." } }

    ::

        >>> show(markup) # doctest: +SKIP

    Initialize any markup from existing markup:

    ::

        >>> markup_1 = markuptools.Markup('foo', direction=Up)
        >>> markup_2 = markuptools.Markup(markup_1, direction=Down)

    ::

        >>> f(markup_1)
        ^ \markup { foo }

    ::

        >>> f(markup_2) # doctest: +SKIP
        _ \markup { foo }

    Attach markup to score components like this:

    ::

        >>> note = Note("c'4")

    ::

        >>> markup = markuptools.Markup(r'\italic { "This is also markup text." }', direction=Up)

    ::

        >>> markup(note)
        Markup((MarkupCommand('italic', ['This is also markup text.']),), direction=Up)(c'4)

    ::

        >>> f(note)
        c'4 ^ \markup { \italic { "This is also markup text." } }

    ::

        >>> show(note) # doctest: +SKIP

    Set `direction` to ``Up``, ``Down``, ``'neutral'``, 
    ``'^'``, ``'_'``, ``'-'`` or None.

    Markup objects are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_contents', '_direction', '_format_slot', '_markup_name')

    _default_positional_input_arguments = (repr(r'\bold { "This is markup text." }'), )

    ### INITIALIZER ###

    def __init__(self, argument, direction=None, markup_name=None):
        from abjad.tools import lilypondparsertools
        from abjad.tools import markuptools
        if not argument:
            contents = None
        elif isinstance(argument, str):
            to_parse = r'\markup {{ {} }}'.format(argument)
            parsed = lilypondparsertools.LilyPondParser()(to_parse)
            if all([isinstance(x, str) for x in parsed.contents]):
                contents = (' '.join(parsed.contents),)
            else:
                contents = tuple(parsed.contents)          
        elif isinstance(argument, markuptools.MarkupCommand):
            contents = (argument,)
        elif isinstance(argument, type(self)):
            contents = tuple(argument._contents)
            direction = direction or argument._direction
            markup_name = markup_name or argument._markup_name
        elif isinstance(argument, (list, tuple)) and 0 < len(argument):
            contents = []
            for arg in argument:
                if isinstance(arg, (str, markuptools.MarkupCommand)):
                    contents.append(arg)
                else:
                    contents.append(str(arg))
            contents = tuple(contents)
        else:
            contents = (str(argument),)
        DirectedMark.__init__(self, direction=direction)
        self._contents = contents
        self._format_slot = 'right'
        self._markup_name = markup_name

    ### SPECIAL METHODS ###

    def __copy__(self, *args):
        return type(self)(
            self._contents,
            direction=self._direction, 
            markup_name=self._markup_name,
            )

    __deepcopy__ = __copy__

    def __eq__(self, expr):
        if isinstance(expr, type(self)):
            if self.lilypond_format == expr.lilypond_format:
                return True
        return False

    def __hash__(self):
        return hash((type(self).__name__, self.contents))

    def __ne__(self, expr):
        return not self == expr

    def __str__(self):
        return self.lilypond_format

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _format_pieces(self):
        return self._get_format_pieces(is_indented=False)

    @property
    def _positional_argument_values(self):
        return (
            self.contents,
            )

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def contents(self):
        r'''Read-only tuple of contents of markup:

        ::

            >>> markup = markuptools.Markup(r'\bold { "This is markup text." }')
            >>> markup.contents
            (MarkupCommand('bold', ['This is markup text.']),)

        Return string
        '''
        return self._contents

    @property
    def indented_lilypond_format(self):
        r'''Read-only indented LilyPond format of markup::

            >>> markup = markuptools.Markup(r'\bold { "This is markup text." }')
            >>> print markup.indented_lilypond_format
            \markup {
                \bold {
                    "This is markup text."
                    }
                }

        Return string.
        '''
        return '\n'.join(self._get_format_pieces(is_indented=True))

    @property
    def lilypond_format(self):
        r'''Read-only LilyPond format of markup::

            >>> markup = markuptools.Markup(r'\bold { "This is markup text." }')
            >>> markup.lilypond_format
            '\\markup { \\bold { "This is markup text." } }'

        Return string.
        '''
        return ' '.join(self._get_format_pieces(is_indented=False))

    @property
    def markup_name(self):
        r'''Read-only name of markup::

            >>> markup = markuptools.Markup(
            ...     r'\bold { allegro ma non troppo }', markup_name='non troppo')

        ::

            >>> markup.markup_name
            'non troppo'

        Return string or none.
        '''
        return self._markup_name

    ### PRIVATE METHODS ###

    def _get_format_pieces(self, is_indented=True):
        indent = ''
        if is_indented:
            indent = '\t'

        direction = ''
        if self.direction is not None:
            direction = stringtools.arg_to_tridirectional_lilypond_symbol(self.direction)

        # None
        if self.contents is None:
            return [r'\markup { }']

        # a single string
        if len(self.contents) == 1 and isinstance(self.contents[0], str):
            content = self.contents[0]
            if '"' in content:
                content = schemetools.format_scheme_value(content)
            if direction:
                return [r'{} \markup {{ {} }}'.format(direction, content)]
            return [r'\markup {{ {} }}'.format(content)]

        # multiple strings or markup commands
        if direction:
            pieces = [r'{} \markup {{'.format(direction)]
        else:
            pieces = [r'\markup {']
        for content in self.contents:
            if isinstance(content, str):
                content = schemetools.format_scheme_value(content)
                pieces.append('{}{}'.format(indent, content))
            else:
                pieces.extend(['{}{}'.format(indent, x) for x in
                    content._get_format_pieces(is_indented=is_indented)])
        pieces.append('{}}}'.format(indent))

        return pieces

    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        result = []
        indent = '\t'
        if not is_indented:
            indent = ''
        result.append('{}(('.format(self._tools_package_qualified_class_name))
        for i, content in enumerate(self.contents):
            comma = ','
            if 1 < len(self.contents) and i == len(self.contents) - 1:
                comma = ''
            if isinstance(content, abctools.AbjadObject):
                pieces = content._get_tools_package_qualified_repr_pieces()
                for piece in pieces[:-1]:
                    result.append('{}{}'.format(indent, piece))
                result.append('{}{}{}'.format(indent, pieces[-1], comma))
            else:
                result.append('{}{!r}{}'.format(indent, content, comma))
        strings = self._keyword_argument_name_value_strings
        if strings:
            result.append('{}),'.format(indent))
            for string in strings[:-1]:
                result.append('{}{},'.format(indent, string))
            result.append('{}{}'.format(indent, strings[-1]))
            result.append('{})'.format(indent))
        else:
            result.append('{}))'.format(indent))
        return result



import copy


def copy_components_and_remove_spanners(components, n=1):
    r'''.. versionadded:: 1.1

    Copy `components` and remove spanners.

    The `components` must be thread-contiguous.

    The steps taken by this function are as follows:

    * Withdraw `components` from spanners.

    * Deep copy unspanned `components`.

    * Reapply spanners to `components`.

    * Return unspanned copy.

    Example:

    ::

        >>> voice = Voice(Measure((2, 8), notetools.make_repeated_notes(2)) * 3)
        >>> pitchtools.set_ascending_named_diatonic_pitches_on_tie_chains_in_expr(voice)
        >>> beam = beamtools.BeamSpanner(voice.leaves[:4])
        >>> f(voice)
        \new Voice {
            {
                \time 2/8
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> result = componenttools.copy_components_and_remove_spanners(voice.leaves[2:4])
        >>> result
        (Note("e'8"), Note("f'8"))

    ::

        >>> new_voice = Voice(result)
        >>> f(new_voice)
        \new Voice {
            e'8
            f'8
        }

    ::

        >>> voice.leaves[2] is new_voice.leaves[0]
        False

    Copy `components` a total of `n` times:

    ::

        >>> result = componenttools.copy_components_and_remove_spanners(voice.leaves[2:4], n=3)
        >>> result
        (Note("e'8"), Note("f'8"), Note("e'8"), Note("f'8"), Note("e'8"), Note("f'8"))

    ::

        >>> new_voice = Voice(result)
        >>> f(new_voice)
        \new Voice {
            e'8
            f'8
            e'8
            f'8
            e'8
            f'8
        }

    Return type equal to input type.
    '''
    from abjad.tools import componenttools

    if n < 1:
        return []

    assert componenttools.all_are_thread_contiguous_components(components)

    result = [component._copy_with_children_and_marks_but_without_spanners() for component in components]
    result = type(components)(result)

    for i in range(n - 1):
        result += copy_components_and_remove_spanners(components)

    return result



from abjad.tools import contexttools
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools import scoretools
from abjad.tools.instrumenttools._KeyboardInstrument import _KeyboardInstrument


# TODO: extend class definition to allow for custom target context in repr
class Piano(_KeyboardInstrument):
    r'''.. versionadded:: 2.0

    Abjad model of the piano::

        >>> piano_staff = scoretools.PianoStaff([Staff("c'8 d'8 e'8 f'8"), Staff("c'4 b4")])

    ::

        >>> instrumenttools.Piano()(piano_staff)
        Piano()(PianoStaff<<2>>)

    ::

        >>> f(piano_staff)
        \new PianoStaff <<
            \set PianoStaff.instrumentName = \markup { Piano }
            \set PianoStaff.shortInstrumentName = \markup { Pf. }
            \new Staff {
                c'8
                d'8
                e'8
                f'8
            }
            \new Staff {
                c'4
                b4
            }
        >>

    ::

        >>> show(piano_staff) # doctest: +SKIP

    The piano targets piano staff context by default.
    '''

    def __init__(self, target_context=None, **kwargs):
        if target_context is None:
            target_context = scoretools.PianoStaff
        _KeyboardInstrument.__init__(self, target_context=target_context, **kwargs)
        self._default_instrument_name = 'piano'
        self._default_performer_names.append('pianist')
        self._default_short_instrument_name = 'pf.'
        self._is_primary_instrument = True
        self.sounding_pitch_of_written_middle_c = pitchtools.NamedChromaticPitch("c'")
        self.primary_clefs = [contexttools.ClefMark('treble'), contexttools.ClefMark('bass')]
        self._copy_primary_clefs_to_all_clefs()
        self._traditional_pitch_range = pitchtools.PitchRange(-39, 48)

    ### READ-ONLY PRIVATE PROPERTIES ###

    # TODO: extend class definition to allow for custom target context in repr
    @property
    def _keyword_argument_names(self):
        return ()

    @property
    def _positional_argument_values(self):
        return ()



from abjad.tools import stringtools
from abjad.tools.lilypondproxytools.LilyPondGrobProxy import LilyPondGrobProxy
from abjad.tools.lilypondproxytools.LilyPondGrobProxyContextWrapper import LilyPondGrobProxyContextWrapper
from abjad.tools.lilypondproxytools._LilyPondComponentPlugIn import _LilyPondComponentPlugIn


class LilyPondGrobOverrideComponentPlugIn(_LilyPondComponentPlugIn):
    '''.. versionadded:: 2.0

    LilyPond grob override component plug-in.
    '''

    ### SPECIAL METHODS ###

    def __getattr__(self, name):
        from abjad import ly
        if name.startswith('_'):
            try:
                return vars(self)[name]
            except KeyError:
                raise AttributeError('"%s" object has no attribute: "%s".' % (
                    self.__class__.__name__, name))
        else:
            camel_name = stringtools.underscore_delimited_lowercase_to_uppercamelcase(name)
            if camel_name in ly.contexts:
                try:
                    return vars(self)['_' + name]
                except KeyError:
                    context = LilyPondGrobProxyContextWrapper()
                    vars(self)['_' + name] = context
                    return context
            elif camel_name in ly.grob_interfaces:
                try:
                    return vars(self)[name]
                except KeyError:
                    vars(self)[name] = LilyPondGrobProxy()
                    return vars(self)[name]
            else:
                return vars(self)[name]

    def __repr__(self):
        body_string = ' '
        skeleton_strings = self._get_skeleton_strings()
        if skeleton_strings:
            # remove 'override__'
            skeleton_strings = [x[10:] for x in skeleton_strings]
            skeleton_strings.sort()
            body_string = ', '.join(skeleton_strings)
        return '%s(%s)' % (self.__class__.__name__, body_string)

    def __setattr__(self, attr, value):
        # make sure attr is valid grob name before setting value #
        attr_value = getattr(self, attr)
        object.__setattr__(self, attr, value)

    ### PRIVATE METHODS ###

    def _get_attribute_tuples(self):
        result = []
        for name, value in vars(self).iteritems():
            if isinstance(value, LilyPondGrobProxy):
                grob_name, grob_proxy = name, value
                for attribute_name, attribute_value in vars(grob_proxy).iteritems():
                    result.append((grob_name, attribute_name, attribute_value))
            else:
                context_name, context_proxy = name.strip('_'), value
                for grob_name, grob_proxy in vars(context_proxy).iteritems():
                    for attribute_name, attribute_value in vars(grob_proxy).iteritems():
                        result.append((context_name, grob_name, attribute_name, attribute_value))
        return tuple(result)

    def _get_skeleton_strings(self):
        skeleton_strings = []
        grob_override_tuples = self._get_attribute_tuples()
        for grob_override_tuple in grob_override_tuples:
            most = '__'.join(grob_override_tuple[:-1])
            value = grob_override_tuple[-1]
            skeleton_string = 'override__%s=%s' % (most, repr(value))
            skeleton_strings.append(skeleton_string)
        return tuple(skeleton_strings)

    def _list_format_contributions(self, contribution_type, is_once=False):
        from abjad.tools.lilypondfiletools._make_lilypond_override_string import _make_lilypond_override_string
        from abjad.tools.lilypondfiletools._make_lilypond_revert_string import _make_lilypond_revert_string
        assert contribution_type in ('override', 'revert')
        result = []
        for attribute_tuple in self._get_attribute_tuples():
            if len(attribute_tuple) == 3:
                context_name = None
                grob_name, attribute_name, attribute_value = attribute_tuple
            elif len(attribute_tuple) == 4:
                context_name, grob_name, attribute_name, attribute_value = attribute_tuple
            else:
                raise ValueError
            if contribution_type == 'override':
                result.append(_make_lilypond_override_string(grob_name, attribute_name,
                    attribute_value, context_name = context_name, is_once = is_once))
            else:
                result.append(
                    _make_lilypond_revert_string(grob_name, attribute_name, context_name=context_name))
        result.sort()
        return result



def report_spanner_format_contributions(component, klass=None):
    r'''.. versionadded:: 1.1

    Report as string format contributions of all spanners attached to `component`::

        >>> staff = Staff("c'8 [ ( d'8 e'8 f'8 ] )")
        >>> trill = spannertools.TrillSpanner(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ( \startTrillSpan
            d'8
            e'8
            f'8 ] ) \stopTrillSpan
        }

    ::

        >>> print spannertools.report_spanner_format_contributions(staff[0])
        BeamSpanner
            _format_right_of_leaf
                [
        SlurSpanner
            _format_right_of_leaf
                (

    Return string.
    '''
    from abjad.tools import spannertools

    result = ''
    locations = ('_format_before_leaf', '_format_right_of_leaf', '_format_after_leaf')
    spanners = list(spannertools.get_spanners_attached_to_component(component, klass))
    spanners.sort(lambda x, y: cmp(x._class_name, y._class_name))
    for spanner in spanners:
        result += '%s\n' % spanner._class_name
        for location in locations:
            contributions = getattr(spanner, location)(component)
            if contributions:
                result += '\t%s\n' % location
                for contribution in contributions:
                    result += '\t\t%s\n' % contribution
    return result



from abjad.tools import notetools


def iterate_notes_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Yield left-to-right notes in `expr`::

        >>> staff = Staff()
        >>> staff.append(Measure((2, 8), "c'8 d'8"))
        >>> staff.append(Measure((2, 8), "e'8 f'8"))
        >>> staff.append(Measure((2, 8), "g'8 a'8"))

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> for note in iterationtools.iterate_notes_in_expr(staff):
        ...     note
        ...
        Note("c'8")
        Note("d'8")
        Note("e'8")
        Note("f'8")
        Note("g'8")
        Note("a'8")

    Use optional `start` and `stop` keyword parameters to control
    start and stop indices of iteration::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, start=3):
        ...     note
        ...
        Note("f'8")
        Note("g'8")
        Note("a'8")

    ::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, start=0, stop=3):
        ...     note
        ...
        Note("c'8")
        Note("d'8")
        Note("e'8")

    ::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, start=2, stop=4):
        ...     note
        ...
        Note("e'8")
        Note("f'8")

    Yield right-to-left notes in `expr`::

        >>> staff = Staff()
        >>> staff.append(Measure((2, 8), "c'8 d'8"))
        >>> staff.append(Measure((2, 8), "e'8 f'8"))
        >>> staff.append(Measure((2, 8), "g'8 a'8"))

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, reverse=True):
        ...     note
        ...
        Note("a'8")
        Note("g'8")
        Note("f'8")
        Note("e'8")
        Note("d'8")
        Note("c'8")

    Use optional `start` and `stop` keyword parameters to control
    indices of iteration::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, reverse=True, start=3):
        ...     note
        ...
        Note("e'8")
        Note("d'8")
        Note("c'8")

    ::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, reverse=True, start=0, stop=3):
        ...     note
        ...
        Note("a'8")
        Note("g'8")
        Note("f'8")

    ::

        >>> for note in iterationtools.iterate_notes_in_expr(staff, reverse=True, start=2, stop=4):
        ...     note
        ...
        Note("f'8")
        Note("e'8")


    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=notetools.Note, reverse=reverse, start=start, stop=stop)



from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_LilyPondParser__marks__KeySignatureMark_01():
    target = Staff([Note("fs'", 1)])
    contexttools.KeySignatureMark('g', 'major')(target[0])

    r'''\new Staff {
        \key g \major
        fs'1
    }
    '''

    parser = LilyPondParser()
    result = parser(target.lilypond_format)
    assert target.lilypond_format == result.lilypond_format and target is not result
    assert 1 == len(contexttools.get_key_signature_marks_attached_to_component(result[0]))



from abjad.tools.beamtools.ComplexBeamSpanner import ComplexBeamSpanner
from abjad.tools import durationtools


class MeasuredComplexBeamSpanner(ComplexBeamSpanner):
    r'''Abjad measured complex beam spanner::

        >>> staff = Staff([Measure((2, 16), "c'16 d'16"), Measure((2, 16), "e'16 f'16")])

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> beamtools.MeasuredComplexBeamSpanner(staff.leaves)
        MeasuredComplexBeamSpanner(c'16, d'16, e'16, f'16)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/16
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #2
                c'16 [
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #1
                d'16
            }
            {
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #2
                e'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #0
                f'16 ]
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Beam leaves in spanner explicitly.

    Group leaves by measures.

    Format top-level `span` beam between measures.

    Return measured complex beam spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None, lone=False, span=1, direction=None):
        ComplexBeamSpanner.__init__(self, components=components, lone=lone, direction=direction)
        self.span = span

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        ComplexBeamSpanner._copy_keyword_args(self, new)
        new.span = self.span

    def _format_before_leaf(self, leaf):
        from abjad.tools import beamtools
        from abjad.tools import componenttools
        from abjad.tools import measuretools
        result = []
        left, right = None, None
        #if leaf.beam.beamable:
        if beamtools.is_beamable_component(leaf):
            if self._is_exterior_leaf(leaf):
                left, right = self._get_left_right_for_exterior_leaf(leaf)
            elif componenttools.get_first_instance_of_klass_in_proper_parentage_of_component(
                leaf, measuretools.Measure) is not None:
                measure = componenttools.get_first_instance_of_klass_in_proper_parentage_of_component(
                    leaf, measuretools.Measure)
                # leaf at beginning of measure
                if measure._is_one_of_my_first_leaves(leaf):
                    assert isinstance(self.span, int)
                    left = self.span
                    right = leaf.written_duration.flag_count
                # leaf at end of measure
                elif measure._is_one_of_my_last_leaves(leaf):
                    assert isinstance(self.span, int)
                    left = leaf.written_duration.flag_count
                    right = self.span
            else:
                left, right = self._get_left_right_for_interior_leaf(leaf)
            if left is not None:
                result.append(r'\set stemLeftBeamCount = #%s' % left)
            if right is not None:
                result.append(r'\set stemRightBeamCount = #%s' % right)
        return result

    ### READ / WRITE PUBLIC PROPERTIES ###

    @apply
    def span():
        def fget(self):
            '''Get top-level beam count::

                >>> staff = Staff([Measure((2, 16), "c'16 d'16"), Measure((2, 16), "e'16 f'16")])
                >>> beam = beamtools.MeasuredComplexBeamSpanner(staff.leaves)
                >>> beam.span
                1

            Set top-level beam count::

                >>> staff = Staff([Measure((2, 16), "c'16 d'16"), Measure((2, 16), "e'16 f'16")])
                >>> beam = beamtools.MeasuredComplexBeamSpanner(staff.leaves)
                >>> beam.span = 2
                >>> beam.span
                2

            Set nonnegative integer.
            '''
            return self._span
        def fset(self, arg):
            assert isinstance(arg, (int, type(None)))
            self._span = arg
        return property(**locals())



from abjad import *


def test_labeltools_label_leaves_in_expr_with_leaf_numbers_01():
    '''Leaf numbers start at 1.'''

    t = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_leaf_numbers(t)

    r'''
    \new Staff {
        c'8
            _ \markup {
                \small
                    1
                }
        d'8
            _ \markup {
                \small
                    2
                }
        e'8
            _ \markup {
                \small
                    3
                }
        f'8
            _ \markup {
                \small
                    4
                }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t1\n\t\t\t}\n\td'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t2\n\t\t\t}\n\te'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t3\n\t\t\t}\n\tf'8\n\t\t_ \\markup {\n\t\t\t\\small\n\t\t\t\t4\n\t\t\t}\n}"


def test_labeltools_label_leaves_in_expr_with_leaf_numbers_02():
    '''Optional markup direction keyword.'''

    t = Staff("c'8 d'8 e'8 f'8")
    labeltools.label_leaves_in_expr_with_leaf_numbers(t, markup_direction=Up)

    r'''
    \new Staff {
        c'8
            ^ \markup {
                \small
                    1
                }
        d'8
            ^ \markup {
                \small
                    2
                }
        e'8
            ^ \markup {
                \small
                    3
                }
        f'8
            ^ \markup {
                \small
                    4
                }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\tc'8\n\t\t^ \\markup {\n\t\t\t\\small\n\t\t\t\t1\n\t\t\t}\n\td'8\n\t\t^ \\markup {\n\t\t\t\\small\n\t\t\t\t2\n\t\t\t}\n\te'8\n\t\t^ \\markup {\n\t\t\t\\small\n\t\t\t\t3\n\t\t\t}\n\tf'8\n\t\t^ \\markup {\n\t\t\t\\small\n\t\t\t\t4\n\t\t\t}\n}"



from abjad import *
from abjad.tools import verticalitytools


def test_labeltools_label_vertical_moments_in_expr_with_pitch_numbers_01():

    score = Score(Staff([]) * 3)
    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    score[0].extend(notes)
    contexttools.ClefMark('alto')(score[1])
    score[1].extend([Note(-5, (1, 4)), Note(-7, (1, 4))])
    contexttools.ClefMark('bass')(score[2])
    score[2].append(Note(-24, (1, 2)))
    labeltools.label_vertical_moments_in_expr_with_pitch_numbers(score)

    r'''
    \new Score <<
        \new Staff {
            c'8
            d'8
                _ \markup {
                    \small
                        \column
                            {
                                2
                                -5
                                -24
                            }
                    }
            e'8
            f'8
                _ \markup {
                    \small
                        \column
                            {
                                5
                                -7
                                -24
                            }
                    }
        }
        \new Staff {
            \clef "alto"
            g4
            f4
                _ \markup {
                    \small
                        \column
                            {
                                4
                                -7
                                -24
                            }
                    }
        }
        \new Staff {
            \clef "bass"
            c,2
                _ \markup {
                    \small
                        \column
                            {
                                0
                                -5
                                -24
                            }
                    }
        }
    >>
    '''

    assert wellformednesstools.is_well_formed_component(score)
    assert score.lilypond_format == '\\new Score <<\n\t\\new Staff {\n\t\tc\'8\n\t\td\'8\n\t\t\t_ \\markup {\n\t\t\t\t\\small\n\t\t\t\t\t\\column\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t2\n\t\t\t\t\t\t\t-5\n\t\t\t\t\t\t\t-24\n\t\t\t\t\t\t}\n\t\t\t\t}\n\t\te\'8\n\t\tf\'8\n\t\t\t_ \\markup {\n\t\t\t\t\\small\n\t\t\t\t\t\\column\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t5\n\t\t\t\t\t\t\t-7\n\t\t\t\t\t\t\t-24\n\t\t\t\t\t\t}\n\t\t\t\t}\n\t}\n\t\\new Staff {\n\t\t\\clef "alto"\n\t\tg4\n\t\tf4\n\t\t\t_ \\markup {\n\t\t\t\t\\small\n\t\t\t\t\t\\column\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t4\n\t\t\t\t\t\t\t-7\n\t\t\t\t\t\t\t-24\n\t\t\t\t\t\t}\n\t\t\t\t}\n\t}\n\t\\new Staff {\n\t\t\\clef "bass"\n\t\tc,2\n\t\t\t_ \\markup {\n\t\t\t\t\\small\n\t\t\t\t\t\\column\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t0\n\t\t\t\t\t\t\t-5\n\t\t\t\t\t\t\t-24\n\t\t\t\t\t\t}\n\t\t\t\t}\n\t}\n>>'





from abjad.tools import mathtools


def is_integer_equivalent_n_tuple(expr, n):
    r'''.. versionadded:: 2.9

    True when `expr` is a tuple of `n` integer-equivalent expressions::

        >>> sequencetools.is_integer_equivalent_n_tuple((2.0, '3', Fraction(4, 1)), 3)
        True

    Otherwise false::

        >>> sequencetools.is_integer_equivalent_n_tuple((2.5, '3', Fraction(4, 1)), 3)
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == n and all([
        mathtools.is_integer_equivalent_expr(x) for x in expr])



from abjad import *


def test_pitchtools_PitchRange_storage_format_01():

    pitch_range = pitchtools.PitchRange('[A0, C8]')

    assert pitch_range.storage_format == "pitchtools.PitchRange(\n\t'[A0, C8]'\n\t)"



from abjad import *


def test_BeamSpanner___eq___01():
    '''Spanner is strict comparator.
    '''

    spanner_1 = beamtools.BeamSpanner()
    spanner_2 = beamtools.BeamSpanner()

    assert not spanner_1 == spanner_2



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import notetools
from abjad.tools import spannertools
from abjad.tools import tuplettools


# TODO: Inspect tietools.add_or_remove_tie_chain_notes_to_achieve_written_duration() carefully.
#       Determine whether behavior is correct with LilyPond multipliers.
def add_or_remove_tie_chain_notes_to_achieve_written_duration(tie_chain, new_written_duration):
    r'''Add or remove `tie_chain` notes to achieve `written_duration`::

        >>> staff = Staff("c'8 [ ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ]
        }

    ::

        >>> tie_chain = tietools.get_tie_chain(staff[0])
        >>> tietools.add_or_remove_tie_chain_notes_to_achieve_written_duration(
        ...     tie_chain, Duration(5, 32))
        TieChain(Note("c'8"), Note("c'32"))

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ~
            c'32 ]
        }

    Return `tie_chain`.

    .. versionchanged:: 2.0
        renamed ``tietools.duration_change()`` to
        ``tietools.add_or_remove_tie_chain_notes_to_achieve_written_duration()``.
    '''
    from abjad.tools import tietools
    from abjad.tools.spannertools._withdraw_components_from_attached_spanners import \
        _withdraw_components_from_attached_spanners

    assert isinstance(tie_chain, tietools.TieChain)
    new_written_duration = durationtools.Duration(new_written_duration)

    if new_written_duration.is_assignable:
        tie_chain[0].written_duration = new_written_duration
        tietools.remove_nonfirst_leaves_in_tie_chain(tie_chain)
    elif new_written_duration.has_power_of_two_denominator:
        durations = notetools.make_notes(0, [new_written_duration])
        for leaf, token in zip(tie_chain, durations):
            leaf.written_duration = token.written_duration
        if len(tie_chain) == len(durations):
            pass
        elif len(durations) < len(tie_chain):
            for leaf in tie_chain[len(durations):]:
                componenttools.remove_component_subtree_from_score_and_spanners([leaf])
        elif len(tie_chain) < len(durations):
            spannertools.destroy_spanners_attached_to_component(tie_chain[0], tietools.TieSpanner)
            difference = len(durations) - len(tie_chain)
            extra_leaves = tie_chain[0] * difference
            _withdraw_components_from_attached_spanners(extra_leaves)
            extra_tokens = durations[len(tie_chain):]
            for leaf, token in zip(extra_leaves, extra_tokens):
                leaf.written_duration = token.written_duration
            if not spannertools.is_component_with_spanner_attached(tie_chain[-1], tietools.TieSpanner):
                tietools.TieSpanner(list(tie_chain))
            componenttools.extend_in_parent_of_component(tie_chain[-1], extra_leaves, grow_spanners=True)
    else:
        durations = notetools.make_notes(0, new_written_duration)
        assert isinstance(durations[0], tuplettools.Tuplet)
        fmtuplet = durations[0]
        new_chain_written = tietools.get_tie_chain(fmtuplet[0]).preprolated_duration
        add_or_remove_tie_chain_notes_to_achieve_written_duration(tie_chain, new_chain_written)
        multiplier = fmtuplet.multiplier
        tuplettools.Tuplet(multiplier, tie_chain.leaves)

    return tietools.get_tie_chain(tie_chain[0])



from abjad import *


def test_tietools_get_tie_chain_01():
    '''Return tuple of all leaves in spanner, if spanned.
    Otherwise return 1-tuple of client.
    '''

    t = Staff(notetools.make_repeated_notes(4))
    tietools.TieSpanner(t[2:])

    r'''
    \new Staff {
        c'8
        d'8
        e'8 ~
        f'8
    }
    '''

    assert tietools.get_tie_chain(t[0]) == tietools.TieChain(t[0])
    assert tietools.get_tie_chain(t[1]) == tietools.TieChain(t[1])
    assert tietools.get_tie_chain(t[2]) == tietools.TieChain((t[2], t[3]))
    assert tietools.get_tie_chain(t[3]) == tietools.TieChain((t[2], t[3]))



from abjad.tools import durationtools
from abjad.tools import selectiontools


def all_are_components_scalable_by_multiplier(components, multiplier):
    '''.. versionadded:: 1.1

    True when `components` are all scalable by `multiplier`::

        >>> components = [Note(0, (1, 8))]
        >>> componenttools.all_are_components_scalable_by_multiplier(components, Multiplier(3, 2))
        True

    Otherwise false::

        >>> components = [Note(0, (1, 8))]
        >>> componenttools.all_are_components_scalable_by_multiplier(components, Multiplier(2, 3))
        False

    Return boolean.

    .. versionchanged:: 2.0
        renamed ``durationtools.are_scalable()`` to
        ``componenttools.all_are_components_scalable_by_multiplier()``.
    '''
    from abjad.tools import leaftools

    # check input
    multiplier = durationtools.Multiplier(multiplier)

    # check components
    for component in components:
        if isinstance(component, leaftools.Leaf):
            candidate_duration = multiplier * component.written_duration
            if not candidate_duration.is_assignable:
                return False

    return True



from abjad import *
import py.test


def test_TreeContainer_index_01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()
    subcontainer = datastructuretools.TreeContainer([leaf_b, leaf_c])
    leaf_d = datastructuretools.TreeNode()
    container = datastructuretools.TreeContainer([leaf_a, subcontainer, leaf_d])

    assert container.index(leaf_a) == 0
    assert container.index(subcontainer) == 1
    assert container.index(leaf_d) == 2

    py.test.raises(ValueError, 'container.index(leaf_b)')
    py.test.raises(ValueError, 'container.index(leaf_c)')



from abjad.tools import chordtools
from abjad.tools import containertools
from abjad.tools import contexttools
from abjad.tools import gracetools
from abjad.tools import leaftools
from abjad.tools import marktools
from abjad.tools import notetools
from abjad.tools import pitchtools
from abjad.tools import stafftools
from abjad.tools import tuplettools
from abjad.tools.abctools import AbjadObject


class GuileProxy(AbjadObject):
    '''Emulates LilyPond music functions.

    Used internally by LilyPondParser.

    Not composer-safe.
    '''

    ### INITIALIZER ###

    def __init__(self, client):
        self.client = client

    ### SPECIAL METHODS ###

    def __call__(self, function_name, args):
        #signature = self.client._current_module[function_name[1:]]['signature'][1:]
        if hasattr(self, function_name[1:]):
            result = getattr(self, function_name[1:])(*args)
            return result
        raise Exception("LilyPondParser can't emulate music function %s." % function_name)


    ### FUNCTION EMULATORS ###


    def acciaccatura(self, music):
        grace = gracetools.GraceContainer(music[:])
        grace.kind = 'acciaccatura'
        return grace


    # afterGrace


    def appoggiatura(self, music):
        grace = gracetools.GraceContainer(music[:])
        grace.kind = 'appoggiatura'
        return grace


    def bar(self, string):
        return marktools.BarLine(string)


    def breathe(self):
        return marktools.LilyPondCommandMark('breathe', 'after')


    def clef(self, string):
        return contexttools.ClefMark(string)


    def grace(self, music):
        return gracetools.GraceContainer(music[:])


    def key(self, notename_pitch, number_list):
        if number_list is None:
            number_list = 'major'
        return contexttools.KeySignatureMark(notename_pitch, number_list)


    def language(self, string):
        if string in self.client._language_pitch_names:
            self.client._pitch_names = self.client._language_pitch_names[string]
        # try reparsing the next note name, if a note name immediately follows
        lookahead = self.client._parser.lookahead
        if lookahead.type == 'STRING':
            if lookahead.value in self.client._pitch_names:
                lookahead.type = 'NOTENAME_PITCH'
                lookahead.value = self.client._pitch_names[lookahead.value]


    def makeClusters(self, music):
        return containertools.Cluster(music[:])


    def mark(self, label):
        if label is None:
            label = '\default'
        return marktools.LilyPondCommandMark('mark %s' % label)


    def oneVoice(self):
        return marktools.LilyPondCommandMark('oneVoice')


    # pitchedTrill


    def relative(self, pitch, music):
        # We should always keep track of the last chord entered.
        # When there are repeated chords (via q),
        # we add the last chord as a key in a _repeated_chords dictionary.
        # Then, we associate a list with the chord "key" in the dict,
        # and append a reference to the repeated chord.

        # Should the referenced chord appear in a relative block,
        # we relativize that chord, and update any repeated chords
        # we've added to its list of referencing chords.

        # The parser's "last_chord" variable will now reflect the 
        # relativized pitches of the original referenced chord,
        # and so any new chord repetitions following the \relative block
        # should result in matching absolute pitches to both the "last_chord"
        # and any other repetitions.

        if self._is_unrelativable(music):
            return music

        def recurse(component, pitch):
            if self._is_unrelativable(component):
                return pitch
            elif isinstance(component, (chordtools.Chord, notetools.Note)):
                pitch = self._make_relative_leaf(component, pitch)
                if component in self.client._repeated_chords:
                    for repeated_chord in self.client._repeated_chords[component]:
                        repeated_chord.written_pitches = component.written_pitches
            elif isinstance(component, containertools.Container):
                for child in component:
                    pitch = recurse(child, pitch)
            return pitch
        
        pitch = recurse(music, pitch)

        self._make_unrelativable(music)

        return music


    def skip(self, duration):
        leaf = skiptools.Skip(duration.duration)
        if duration.multiplier is not None:
            leaf.duration_multiplier = duration.multiplier
        return leaf


    def slashedGraceContainer(self, music):
        grace = gracetools.GraceContainer(music[:])
        grace.kind = 'slashedGrace'
        return grace


    def time(self, number_list, fraction):
        n, d = fraction.numerator, fraction.denominator
        return contexttools.TimeSignatureMark((n, d), target_context=stafftools.Staff)


    def times(self, fraction, music):
        n, d  = fraction.numerator, fraction.denominator
        if not isinstance(music, contexttools.Context) and not isinstance(music, leaftools.Leaf):
            return tuplettools.Tuplet((n, d), music[:])
        return tuplettools.Tuplet((n, d), [music])

    def transpose(self, from_pitch, to_pitch, music):
        from abjad.tools import lilypondparsertools

        self._make_unrelativable(music)

        def recurse(music):
            key_signatures = contexttools.get_key_signature_marks_attached_to_component(music)
            if key_signatures:
                for x in key_signatures:
                    tonic = pitchtools.NamedChromaticPitch(x.tonic, 4)
                    x.tonic = lilypondparsertools.lilypond_enharmonic_transpose(
                        from_pitch, to_pitch, tonic).named_chromatic_pitch_class
            if isinstance(music, notetools.Note):
                music.written_pitch = lilypondparsertools.lilypond_enharmonic_transpose(
                    from_pitch, to_pitch, music.written_pitch)
            elif isinstance(music, chordtools.Chord):
                for note_head in music.note_heads:
                    note_head.written_pitch = lilypondparsertools.lilypond_enharmonic_transpose(
                        from_pitch, to_pitch, note_head.written_pitch)
            elif isinstance(music, containertools.Container):
                for x in music:
                    recurse(x)

        recurse(music)

        return music


    # transposition


    # tweak


    def voiceOne(self):
        return marktools.LilyPondCommandMark('voiceOne')


    def voiceFour(self):    
        return marktools.LilyPondCommandMark('voiceTwo')


    def voiceThree(self):
        return marktools.LilyPondCommandMark('voiceThree')


    def voiceTwo(self):
        return marktools.LilyPondCommandMark('voiceFour')


    ### HELPER FUNCTIONS ###


    def _is_unrelativable(self, music):
        annotations = marktools.get_annotations_attached_to_component(music)
        if 'UnrelativableMusic' in [x.name for x in annotations]:
            return True
        return False


    def _make_relative_leaf(self, leaf, pitch):
        if self._is_unrelativable(leaf):
            return pitch
        elif isinstance(leaf, notetools.Note):
            pitch = self._to_relative_octave(leaf.written_pitch, pitch)
            leaf.written_pitch = pitch
        elif isinstance(leaf, chordtools.Chord):
            # TODO: This is not ideal w/r/t post events as LilyPond does not sort chord contents
            chord_pitches = self.client._chord_pitch_orders[leaf]
            for i, chord_pitch in enumerate(chord_pitches):
                pitch = self._to_relative_octave(chord_pitch, pitch)
                chord_pitches[i] = pitch
            leaf.written_pitches = chord_pitches
            pitch = min(leaf.written_pitches)
        return pitch


    def _make_unrelativable(self, music):
        if not self._is_unrelativable(music):
            marktools.Annotation('UnrelativableMusic')(music)
        

    def _to_relative_octave(self, pitch, reference):
        if pitch.chromatic_pitch_class_number > reference.chromatic_pitch_class_number:
            up_pitch = pitchtools.NamedChromaticPitch(
                pitch.chromatic_pitch_class_name, reference.octave_number)
            down_pitch = pitchtools.NamedChromaticPitch(
                pitch.chromatic_pitch_class_name, reference.octave_number - 1)
            up_octave, down_octave = up_pitch.octave_number, down_pitch.octave_number
        else:
            up_pitch = pitchtools.NamedChromaticPitch(
                pitch.chromatic_pitch_class_name, reference.octave_number + 1)
            down_pitch = pitchtools.NamedChromaticPitch(
                pitch.chromatic_pitch_class_name, reference.octave_number)
            up_octave, down_octave = up_pitch.octave_number, down_pitch.octave_number

        if abs(float(up_pitch.named_diatonic_pitch) - float(reference.named_diatonic_pitch)) \
            < abs(float(down_pitch.named_diatonic_pitch) - float(reference.named_diatonic_pitch)):
            #pitch = up_pitch + (12 * (pitch.octave_number - 3))
            pitch = pitchtools.NamedChromaticPitch(
                up_pitch.named_chromatic_pitch_class, up_octave + pitch.octave_number - 3)
        else:
            #pitch = down_pitch + (12 * (pitch.octave_number - 3))
            pitch = pitchtools.NamedChromaticPitch(
                down_pitch.named_chromatic_pitch_class, down_octave + pitch.octave_number - 3)

        return pitch




from abjad.tools import componenttools


def move_marks(donor, recipient):
    r'''.. versionadded 2.9

    Move marks from `donor` component to `recipient` component::

        >>> staff = Staff(r'\clef "bass" c \staccato d e f')

    ::

        >>> f(staff)
        \new Staff {
            \clef "bass"
            c4 -\staccato
            d4
            e4
            f4
        }

    ::

        >>> marktools.move_marks(staff[0], staff[2])
        [Articulation('staccato')(e4), ClefMark('bass')(e4)]

    ::

        >>> f(staff)
        \new Staff {
            c4
            d4
            \clef "bass"
            e4 -\staccato
            f4
        }

    Return list of marks moved.
    '''
    from abjad.tools import marktools

    assert isinstance(donor, componenttools.Component)
    assert isinstance(recipient, componenttools.Component)

    result = []
    for mark in marktools.get_marks_attached_to_component(donor):
        result.append(mark.attach(recipient))

    return result



import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class BoundedObject(AbjadObject):
    r'''.. versionadded:: 2.10

    Bounded object mix-in.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def is_closed(self):
        return self.is_left_closed and self.is_right_closed

    @property
    def is_half_closed(self):
        return not self.is_left_closed == self.is_right_closed

    @property
    def is_half_open(self):
        return not self.is_left_open == self.is_right_open

    @property
    def is_open(self):
        return not self.is_left_closed and not self.is_right_closed

    ### READ / WRITE PUBLIC PROPERTIES ### 

    @apply
    def is_left_closed():
        def fget(self):
            return self._is_left_closed
        def fset(self, is_left_closed):
            assert isinstance(is_left_closed, bool), is_left_closed
            self._is_left_closed = is_left_closed
        return property(**locals())

    @apply
    def is_left_open():
        def fget(self):
            return not self.is_left_closed
        def fset(self, is_left_open):
            assert isinstance(is_left_open, bool), is_left_open
            self._is_left_closed = not is_left_open
        return property(**locals())

    @apply
    def is_right_closed():
        def fget(self):
            return self._is_right_closed
        def fset(self, is_right_closed):
            assert isinstance(is_right_closed, bool), is_right_closed
            self._is_right_closed = is_right_closed
        return property(**locals())

    @apply
    def is_right_open():
        def fget(self):
            return not self.is_right_closed
        def fset(self, is_right_open):
            assert isinstance(is_right_open, bool), is_right_open
            self._is_right_closed = not is_right_open
        return property(**locals())



def add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration(tie_chain, multiplier):
    r'''Add or remove `tie_chain` notes to achieve scaled written duration::

        >>> staff = Staff("c'8 [ ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ]
        }

    ::
    
        >>> tie_chain = tietools.get_tie_chain(staff[0])
        >>> tietools.add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration(
        ...    tie_chain, Fraction(5, 4))
        TieChain(Note("c'8"), Note("c'32"))

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ~
            c'32 ]
        }

    Return `tie_chain`.

    .. versionchanged:: 2.0
        renamed ``tietools.duration_scale()`` to
        ``tietools.add_or_remove_tie_chain_notes_to_achieve_scaled_written_duration()``.
    '''
    from abjad.tools import tietools

    # find new tie chain written duration
    new_written_duration = multiplier * tie_chain.written_duration

    # adjust tie chain and return tie chain
    return tietools.add_or_remove_tie_chain_notes_to_achieve_written_duration(tie_chain, new_written_duration)



def import_structured_package(path, namespace, package_root_name='abjad'):
    r'''.. versionadded:: 2.9

    Import public names from `path` into `namespace`.

    This is the custom function that all Abjad packages use to import 
    public classes and functions on startup.

    The function will work for any package laid out like Abjad packages.

    Set `package_root_name` to the root any Abjad-like package structure.
    
    Return none.
    '''
    from abjad.tools.importtools._import_public_names_from_path_into_namespace import \
        _import_public_names_from_path_into_namespace

    _import_public_names_from_path_into_namespace(path, namespace, package_root_name)



from abjad.tools import mathtools
from abjad.tools.abctools import AbjadObject


class _Job(AbjadObject):

    ### INITIALIZER ###

    def __init__(self, number):
        self.number = number

    ### SPECIAL METHODS ###

    def __call__(self):
        self.result = [x for x in mathtools.yield_all_compositions_of_integer(self.number)]

    def __repr__(self):
        return '{}({})'.format(self._class_name, self.number)



import fractions
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools import tuplettools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class TupletMonadRhythmMaker(RhythmMaker):
    r'''.. versionadded:: 2.10

    Tuplet monad rhythm-maker::

        >>> maker = rhythmmakertools.TupletMonadRhythmMaker()

    ::

        >>> divisions = [(1, 5), (1, 4), (1, 6), (7, 9)]
        >>> tuplet_lists = maker(divisions)
        >>> tuplets = sequencetools.flatten_sequence(tuplet_lists)

    ::

        >>> staff = Staff(tuplets)

    ::

        >>> f(staff)
        \new Staff {
            \times 4/5 {
                c'4
            }
            {
                c'4
            }
            \times 2/3 {
                c'4
            }
            \times 8/9 {
                c'2..
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self, beam_each_cell=False, beam_cells_together=False):
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together
            )

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        result = []
        for division in divisions:
            monad = self._make_monad(division)
            result.append([monad])
        return result

    ### PRIVATE METHODS ###

    def _make_monad(self, division):
        numerator, talea_denominator = division
        power_of_two_denominator = mathtools.greatest_power_of_two_less_equal(talea_denominator)
        duration = fractions.Fraction(abs(numerator), talea_denominator)
        power_of_two_duration = fractions.Fraction(abs(numerator), power_of_two_denominator)
        power_of_two_division = (numerator, power_of_two_denominator) 
        tuplet_multiplier = duration / power_of_two_duration
        leaves = leaftools.make_leaves([0], [power_of_two_division])
        tuplet = tuplettools.Tuplet(tuplet_multiplier, leaves)
        return tuplet



from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import leaftools
from abjad.tools import markuptools
from abjad.tools import notetools
from abjad.tools import pitchtools


def label_leaves_in_expr_with_melodic_diatonic_intervals(expr, markup_direction=Up):
    r""".. versionadded:: 2.0

    Label leaves in `expr` with melodic diatonic intervals::

        >>> notes = notetools.make_notes([0, 25, 11, -4, -14, -13, 9, 10, 6, 5], [Duration(1, 8)])
        >>> staff = Staff(notes)
        >>> labeltools.label_leaves_in_expr_with_melodic_diatonic_intervals(staff)

    ::

        >>> f(staff)
        \new Staff {
            c'8 ^ \markup { +aug15 }
            cs'''8 ^ \markup { -M9 }
            b'8 ^ \markup { -aug9 }
            af8 ^ \markup { -m7 }
            bf,8 ^ \markup { +aug1 }
            b,8 ^ \markup { +m14 }
            a'8 ^ \markup { +m2 }
            bf'8 ^ \markup { -dim4 }
            fs'8 ^ \markup { -aug1 }
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.
    """

    for note in iterationtools.iterate_notes_in_expr(expr):
        thread_iterator = iterationtools.iterate_thread_from_component(note, leaftools.Leaf)
        try:
            thread_iterator.next()
            next_leaf = thread_iterator.next()
            if isinstance(next_leaf, notetools.Note):
                mdi = pitchtools.calculate_melodic_diatonic_interval(
                    note, next_leaf)
                markuptools.Markup(mdi, markup_direction)(note)
        except StopIteration:
            pass



import types
from abjad.tools import selectiontools


def all_are_contiguous_components_in_same_parent(expr, klasses=None, allow_orphans=True):
    '''.. versionadded:: 1.1

    True when elements in `expr` are all contiguous components in same parent.
    Otherwise false::

        >>> staff = Staff("c'8 d'8 e'8")
        >>> componenttools.all_are_contiguous_components_in_same_parent(staff.leaves)
        True

    True when elements in `expr` are all contiguous `klasses` in same parent.
    Otherwise false::

        >>> staff = Staff("c'8 d'8 e'8")
        >>> componenttools.all_are_contiguous_components_in_same_parent(staff.leaves, klasses=Note)
        True

    Return boolean.
    '''
    from abjad.tools import componenttools

    if not isinstance(expr, (list, tuple, types.GeneratorType, selectiontools.Selection)):
        return False

    if klasses is None:
        klasses = componenttools.Component

    if len(expr) == 0:
        return True

    first = expr[0]
    if not isinstance(first, klasses):
        return False

    first_parent = first._parent
    if first_parent is None:
        if allow_orphans:
            orphan_components = True
        else:
            return False

    same_parent = True
    strictly_contiguous = True

    prev = first
    for cur in expr[1:]:
        if not isinstance(cur, klasses):
            return False
        if not cur.parentage.is_orphan:
            orphan_components = False
        if not cur._parent is first_parent:
            same_parent = False
        if not componenttools.is_immediate_temporal_successor_of_component(prev, cur):
            strictly_contiguous = False
        if (not allow_orphans or (allow_orphans and not orphan_components)) and \
            (not same_parent or not strictly_contiguous):
            return False
        prev = cur

    return True



from abjad.tools import iotools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
from abjad.tools.developerscripttools.CleanScript import CleanScript
import argparse
import os


class SvnUpdateScript(DirectoryScript):
    '''Run `svn up` on various Abjad paths:

    ::

        bash$ ajv svn up -h
        usage: svn-update [-h] [--version] [-C] [-P PATH | -E | -M | -R]

        "svn update" various paths.

        optional arguments:
          -h, --help            show this help message and exit
          --version             show program's version number and exit
          -C, --clean           remove .pyc files and __pycache__ directories before
                                updating
          -P PATH, --path PATH  update the path PATH
          -E, --abjad.tools    update Abjad abjad.tools directory
          -M, --mainline        update Abjad mainline directory
          -R, --root            update Abjad root directory

        If no path flag is specified, the current directory will be updated.

    It is usually most useful to run the script with the `--clean` flag, in case
    there are incoming deletes, as `svn` will not delete directories containing
    unversioned files, such as .pyc:

    ::

        bash$ ajv svn up -C -R

    Return `SvnUpdateScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'up'

    @property
    def long_description(self):
        return 'If no path flag is specified, the current directory will be updated.'

    @property
    def scripting_group(self):
        return 'svn'

    @property
    def short_description(self):
        return '"svn update" various paths.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):

        if args.clean:
            clean_args = [args.path]
            clean_script = CleanScript()
            clean_script(clean_args)

        print 'Updating...'
        iotools.spawn_subprocess('svn update {}'.format(args.path))

    def setup_argument_parser(self, parser):

        from abjad import ABJCFG

        parser.add_argument('-C', '--clean',
            action='store_true',
            help='remove .pyc files and __pycache__ directories before updating'
            )

        group = parser.add_mutually_exclusive_group()

        group.add_argument('-P', '--path',
            dest='path',
            help='update the path PATH',
            #metavar='X',
            type=self._validate_path,
            )

        group.add_argument('-E', '--experimental',
            action='store_const',
            const=ABJCFG.ABJAD_EXPERIMENTAL_PATH,
            dest='path',
            help='update Abjad abjad.tools directory',
            )

        group.add_argument('-M', '--mainline',
            action='store_const',
            const=ABJCFG.ABJAD_PATH,
            dest='path',
            help='update Abjad mainline directory',
            )

        group.add_argument('-R', '--root',
            action='store_const',
            const=ABJCFG.ABJAD_ROOT_PATH,
            dest='path',
            help='update Abjad root directory',
            )

        parser.set_defaults(path=os.path.abspath(os.path.curdir))



from abjad import *


def test_Tuplet_timespan_in_seconds_01():

    staff = Staff(r"c'4 d'4 \times 2/3 { e'4 f'4 g'4 }")
    score = Score([staff])
    contexttools.TempoMark((1, 4), 60)(staff.leaves[0])

    r'''
    \new Score <<
        \new Staff {
            \tempo 4=60
            c'4
            d'4
            \times 2/3 {
                e'4
                f'4
                g'4
            }
        }
    >>
    '''

    assert staff.timespan_in_seconds == timespantools.Timespan(0, 4)
    assert staff[0].timespan_in_seconds == timespantools.Timespan(0, 1)
    assert staff[1].timespan_in_seconds == timespantools.Timespan(1, 2)
    assert staff[-1].timespan_in_seconds == timespantools.Timespan(2, 4)



from abjad import *


def test_tuplettools_change_fixed_duration_tuplets_in_expr_to_tuplets_01():

    staff = Staff(2 * tuplettools.FixedDurationTuplet((2, 8), "c'8 d'8 e'8"))
    assert all([isinstance(x, tuplettools.FixedDurationTuplet) for x in staff])

    tuplettools.change_fixed_duration_tuplets_in_expr_to_tuplets(staff)
    assert all([x._class_name == 'Tuplet' for x in staff])



from abjad.tools import iterationtools
from abjad.tools import leaftools
from abjad.tools import notetools
from abjad.tools import spannertools
from abjad.tools import tietools
from abjad.tools.wellformednesstools.Check import Check


class MispitchedTieCheck(Check):

    def _run(self, expr):
        '''Check for mispitched notes.
        Do not check tied rests or skips.
        Implement chord-checking later.
        '''
        violators = []
        total = 0
        for leaf in iterationtools.iterate_components_in_expr(expr, notetools.Note):
            total += 1
            spanners = spannertools.get_spanners_attached_to_component(
                leaf, tietools.TieSpanner)
            if spanners:
                spanner = spanners.pop()
                if not spanner._is_my_last_leaf(leaf):
                    next_leaf = leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1)
                    if next_leaf:
                        if leaf.written_pitch != next_leaf.written_pitch:
                            violators.append(leaf)
        return violators, total



from abjad.tools import containertools
from abjad.tools import leaftools
from abjad.tools import selectiontools
from abjad.tools import spannertools


def iterate_topmost_tie_chains_and_components_in_expr(expr):
    r'''Iterate topemost tie chains and components forward in `expr`::

        >>> string = r"c'8 ~ c'32 d'8 ~ d'32 \times 2/3 { e'8 f'8 g'8 } a'8 ~ a'32 b'8 ~ b'32"
        >>> staff = Staff(string)

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'32
            d'8 ~
            d'32
            \times 2/3 {
                e'8
                f'8
                g'8
            }
            a'8 ~
            a'32
            b'8 ~
            b'32
        }

    ::

        >>> for x in tietools.iterate_topmost_tie_chains_and_components_in_expr(staff):
        ...     x
        ...
        TieChain(Note("c'8"), Note("c'32"))
        TieChain(Note("d'8"), Note("d'32"))
        Tuplet(2/3, [e'8, f'8, g'8])
        TieChain(Note("a'8"), Note("a'32"))
        TieChain(Note("b'8"), Note("b'32"))

    Raise tie chain error on overlapping tie chains.

    Return generator.

    .. versionchanged:: 2.0
        renamed ``iterate.chained_contents()`` to
        ``tietools.iterate_topmost_tie_chains_and_components_in_expr()``.
    '''
    from abjad.tools import tietools

    if isinstance(expr, leaftools.Leaf):
        if len(tietools.get_tie_chain(expr)) == 1:
            yield tietools.get_tie_chain(expr)
        else:
            raise TieChainError('can not have only one leaf in tie chain.')
    elif isinstance(expr, (list, containertools.Container, selectiontools.Selection)):
        for component in expr:
            if isinstance(component, leaftools.Leaf):
                tie_spanners = spannertools.get_spanners_attached_to_component(
                    component, tietools.TieSpanner)
                if not tie_spanners or tuple(tie_spanners)[0]._is_my_last_leaf(component):
                    yield tietools.get_tie_chain(component)
            elif isinstance(component, containertools.Container):
                yield component
    else:
        raise ValueError('input must be iterable: {!r}.'.format(expr))



from abjad.tools.datastructuretools.TreeNode import TreeNode


class ReSTTOCItem(TreeNode):
    '''An ReST TOC item:

    ::

        >>> item = documentationtools.ReSTTOCItem(text='api/index')
        >>> item
        ReSTTOCItem(
            text='api/index'
            )

    ::

        >>> print item.rest_format
        api/index

    Return `ReSTTOCItem` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, name=None, text=None):
        TreeNode.__init__(self, name)
        self.text = text

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _rest_format_contributions(self):
        return [self.text]

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def rest_format(self):
        return '\n'.join(self._rest_format_contributions)

    ### READ/WRITE PUBLIC PROPERTIES ###

    @apply
    def text():
        def fget(self):
            return self._text
        def fset(self, arg):
            assert isinstance(arg, str)
            arg = arg.strip()
            assert len(arg)
            self._text = arg
        return property(**locals())




import abc
import inspect
import logging
import os
from ply import lex
from ply import yacc
from abjad.tools.abctools.AbjadObject import AbjadObject


class Parser(AbjadObject):
    '''Abstract base class for Abjad parsers.

    Rules objects for lexing and parsing must be defined by overriding the
    abstract properties `lexer_rules_object` and `parser_rules_object`.

    For most parsers these properties should simply return `self`.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_debug', '_lexer', '_logger', '_parser')

    ### INITIALIZER ###

    def __init__(self, debug=False):

        self._debug = bool(debug)
        self._lexer = None
        self._parser = None

        if self.debug:
            logging.basicConfig(
                level = logging.DEBUG,
                filename = self.logger_path,
                filemode = 'w',
                format = '%(filename)10s:%(lineno)8d:%(message)s'
                )
            self._logger = logging.getLogger()
        else:
            self._logger = logging.getLogger()
            self._logger.addHandler(logging.NullHandler())

        self._lexer = lex.lex(
            debug=self.debug,
            debuglog=self.logger,
            object=self.lexer_rules_object
            )

        self._parser = yacc.yacc(
            debug=self.debug,
            debuglog=self.logger,
            module=self.parser_rules_object,
            outputdir=self.output_path,
            picklefile=self.pickle_path,
            )

    ### SPECIAL METHODS ###

    def __call__(self, input_string):
        '''Parse `input_string` and return result.'''

        if hasattr(self, '_setup'):
            self._setup()

        if self.debug:
            result = self.parser.parse_debug(
                input_string,
                lexer=self.lexer,
                debug=self.logger)
        else:
            result = self.parser.parse(
                input_string,
                lexer=self.lexer)

        if hasattr(self, '_cleanup'):
            result = self._cleanup(result)

        return result

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def debug(self):
        '''True if the parser runs in debugging mode.'''
        return self._debug

    @property
    def lexer(self):
        '''The parser's PLY Lexer instance.'''
        return self._lexer

    @abc.abstractproperty
    def lexer_rules_object(self):
        '''The object containing the parser's lexical rule definitions.'''
        raise NotImplemented

    @property
    def logger(self):
        '''The parser's Logger instance.'''
        return self._logger

    @property
    def logger_path(self):
        '''The output path for the parser's logfile.'''
        return os.path.join(self.output_path, 'parselog.txt')

    @property
    def output_path(self):
        '''The output path for files associated with the parser.'''
        klass_path = inspect.getfile(self.__class__)
        return klass_path.rpartition(os.path.sep)[0]

    @property
    def parser(self):
        '''The parser's PLY LRParser instance.'''
        return self._parser

    @abc.abstractproperty
    def parser_rules_object(self):
        '''The object containing the parser's syntactical rule definitions.'''
        raise NotImplemented

    @property
    def pickle_path(self):
        '''The output path for the parser's pickled parsing tables.'''
        return os.path.join(self.output_path, '_parsetab.pkl')

    ### PUBLIC METHODS ###

    def tokenize(self, input_string):
        '''Tokenize `input string` and print results.'''
        self.lexer.input(input_string)
        for token in self.lexer:
            print token



def yield_components_grouped_by_duration(components):
    r'''.. versionadded:: 2.0

    Example 1. Yield topmost components grouped by prolated duration::

        >>> staff = Staff(r"\times 2/3 { c'4 c'4 c'8 c'16 c'16 } c'16 c'16 c'8 c'8")

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'4
                c'4
                c'8
                c'16
                c'16
            }
            c'16
            c'16
            c'8
            c'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    ::

        >>> for x in componenttools.yield_components_grouped_by_duration(staff):
        ...     x
        ...
        (Tuplet(2/3, [c'4, c'4, c'8, c'16, c'16]),)
        (Note("c'16"), Note("c'16"))
        (Note("c'8"), Note("c'8"))

    Example 2. Yield topmost components grouped by prolated duration.

    Note that function treats input as a flat sequence and 
    attempts no navigation of the score tree.
    But it's possible to group components lower in the score tree by passing
    the output of a component iterator as input to this function::

        >>> staff = Staff(r"\times 2/3 { c'4 c'4 c'8 c'16 c'16 } c'16 c'16 c'8 c'8")

    ::

        >>> leaves = iterationtools.iterate_leaves_in_expr(staff)
        >>> for x in componenttools.yield_components_grouped_by_duration(leaves):
        ...     x
        (Note("c'4"), Note("c'4"))
        (Note("c'8"),)
        (Note("c'16"), Note("c'16"))
        (Note("c'16"), Note("c'16"))
        (Note("c'8"), Note("c'8"))

    Return generator.
    
    .. note:: Might be best to add ``in_sequence`` or ``topmost`` to the name
        of this function.
    '''

    current_group = []
    for component in components:
        if current_group:
            previous_component = current_group[-1]
            previous_duration = previous_component.prolated_duration
            current_duration = component.prolated_duration
            if current_duration == previous_duration:
                current_group.append(component)
            else:
                yield tuple(current_group)
                current_group = []
                current_group.append(component)
        else:
            current_group.append(component)
    if current_group:
        yield tuple(current_group)



from abjad import *


def test_lilypondfiletools_make_time_signature_context_block_01():

    context_block = lilypondfiletools.make_time_signature_context_block()
    
    r'''
    \context {
        \type Engraver_group
        \name TimeSignatureContext
        \consists Axis_group_engraver
        \consists Time_signature_engraver
        \override TimeSignature #'X-extent = #'(0 . 0)
        \override TimeSignature #'X-offset = #ly:self-alignment-interface::x-aligned-on-self
        \override TimeSignature #'Y-extent = #'(0 . 0)
        \override TimeSignature #'break-align-symbol = ##f
        \override TimeSignature #'break-visibility = #end-of-line-invisible
        \override TimeSignature #'font-size = #3
        \override TimeSignature #'self-alignment-X = #center
        \override VerticalAxisGroup #'default-staff-staff-spacing = #'((basic_distance . 0) (minimum_distance . 0) (padding . 4) (stretchability . 0))
    }
    '''

    assert context_block.lilypond_format == "\\context {\n\t\\type Engraver_group\n\t\\name TimeSignatureContext\n\t\\consists Axis_group_engraver\n\t\\consists Time_signature_engraver\n\t\\override TimeSignature #'X-extent = #'(0 . 0)\n\t\\override TimeSignature #'X-offset = #ly:self-alignment-interface::x-aligned-on-self\n\t\\override TimeSignature #'Y-extent = #'(0 . 0)\n\t\\override TimeSignature #'break-align-symbol = ##f\n\t\\override TimeSignature #'break-visibility = #end-of-line-invisible\n\t\\override TimeSignature #'font-size = #3\n\t\\override TimeSignature #'self-alignment-X = #center\n\t\\override VerticalAxisGroup #'default-staff-staff-spacing = #'((basic_distance . 0) (minimum_distance . 0) (padding . 4) (stretchability . 0))\n}"



from abjad.tools.abctools.ImmutableAbjadObject import ImmutableAbjadObject


class OrdinalConstant(ImmutableAbjadObject):
    r'''.. versionadded:: 1.0

    Ordinal constant.

    Initialize with `dimension`, `value` and `representation`::

        >>> Left = datastructuretools.OrdinalConstant('x', -1, 'Left')
        >>> Left
        Left

    ::

        >>> Right = datastructuretools.OrdinalConstant('x', 1, 'Right')
        >>> Right
        Right

    ::

        >>> Left < Right
        True

    Comparing like-dimensioned ordinal constants is allowed::

        >>> Up = datastructuretools.OrdinalConstant('y', 1, 'Up')
        >>> Up
        Up

    ::

        >>> Down = datastructuretools.OrdinalConstant('y', -1, 'Down')
        >>> Down
        Down

    ::

        >>> Down < Up
        True

    Comparing differently dimensioned ordinal constants raises an exception::

        >>> import py.test
    
    ::

        >>> bool(py.test.raises(Exception, 'Left < Up'))
        True

    The ``Left``, ``Right``, ``Center``, ``Up`` and ``Down`` constants shown here 
    load into Python's built-in namespace on Abjad import.

    These four objects can be used as constant values supplied to keywords.

    This behavior is similar to True, False and None.
    
    Ordinal constants are immutable.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = (
        '_dimension',
        '_representation', 
        '_value',
        )
   
    ### INITIALIZER ###

    def __new__(klass, dimension, value, representation):
        assert isinstance(dimension, str), repr(dimension)
        assert isinstance(representation, str), repr(representation)
        self = object.__new__(klass)
        self._dimension = dimension
        self._value = value
        self._representation = representation
        return self

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        if isinstance(expr, type(self)):
            if self._dimension == expr._dimension:
                return self._value == expr._value
        return False

    def __ge__(self, expr):
        self._check_comparator(expr)
        return self._value >= expr._value

    def __getnewargs__(self):
        return self._positional_argument_values

    def __gt__(self, expr):
        self._check_comparator(expr)
        return self._value > expr._value

    def __le__(self, expr):
        self._check_comparator(expr)
        return self._value <= expr._value

    def __lt__(self, expr):
        self._check_comparator(expr)
        return self._value < expr._value

    def __repr__(self):
        return self._representation

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _positional_argument_values(self):
        return (
            self._dimension,
            self._value,
            self._representation,
            )

    ### PRIVATE METHODS ###

    # can only compare like-dimensioned ordinal constants
    def _check_comparator(self, expr):
        if not isinstance(expr, type(self)) or \
            not self._dimension == expr._dimension:
            raise Exception('can only compare like-dimensioned ordinal constants.')

    # ordinal constants appear as constants in storage format
    def _get_tools_package_qualified_repr_pieces(self, is_indented=True):
        return [repr(self)]



from abjad.tools import measuretools


def iterate_measures_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate measures forward in `expr`::

        >>> staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 || 2/8 g'8 a'8 |")

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
        }

    ::

        >>> for measure in iterationtools.iterate_measures_in_expr(staff):
        ...     measure
        ...
        Measure(2/8, [c'8, d'8])
        Measure(2/8, [e'8, f'8])
        Measure(2/8, [g'8, a'8])

    Use the optional `start` and `stop` keyword parameters to control
    the start and stop indices of iteration. ::

        >>> for measure in iterationtools.iterate_measures_in_expr(
        ...     staff, start=1):
        ...     measure
        ...
        Measure(2/8, [e'8, f'8])
        Measure(2/8, [g'8, a'8])

    ::

        >>> for measure in iterationtools.iterate_measures_in_expr(
        ...     staff, start=0, stop=2):
        ...     measure
        ...
        Measure(2/8, [c'8, d'8])
        Measure(2/8, [e'8, f'8])

    Iterate measures backward in `expr`::

        >>> for measure in iterationtools.iterate_measures_in_expr(
        ...     staff, reverse=True):
        ...     measure
        ...
        Measure(2/8, [g'8, a'8])
        Measure(2/8, [e'8, f'8])
        Measure(2/8, [c'8, d'8])

    Use the optional `start` and `stop` keyword parameters
    to control indices of iteration. ::

        >>> for measure in iterationtools.iterate_measures_in_expr(
        ...     staff, start=1, reverse=True):
        ...     measure
        ...
        Measure(2/8, [e'8, f'8])
        Measure(2/8, [c'8, d'8])

    ::

        >>> for measure in iterationtools.iterate_measures_in_expr(
        ...     staff, start=0, stop=2, reverse=True):
        ...     measure
        ...
        Measure(2/8, [g'8, a'8])
        Measure(2/8, [e'8, f'8])

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=measuretools.Measure, reverse=reverse, start=start, stop=stop)



import fractions


def calculate_density_of_releases_in_interval(intervals, interval):
    '''Calculate the number of releases in `interval` divided by the duration of `interval`.

    Return Fraction.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    return fractions.Fraction(len(tree.find_intervals_stopping_within_interval(interval))) \
        / interval.duration



from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import notetools
from abjad.tools import spannertools
from abjad.tools import tuplettools


def tie_chain_to_tuplet_with_ratio(tie_chain, proportions, is_diminution=True, dotted=True):
    r'''.. versionadded:: 2.0

    Example 1a. Change `tie_chain` to augmented tuplet with proportions ``[1]``.
    Avoid dots::

        >>> staff = Staff("c'8 [ ~ c'16 c'16 ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ~
            c'16
            c'16 ]
        }

    ::

        >>> tie_chain = tietools.get_tie_chain(staff[0])
        >>> tietools.tie_chain_to_tuplet_with_ratio(
        ...     tie_chain, [1], is_diminution=False, dotted=False)
        FixedDurationTuplet(3/16, [c'8])

    ::

        >>> f(staff)
        \new Staff {
            \fraction \times 3/2 {
                c'8 [
            }
            c'16 ]
        }

    Exampl 1b. Change `tie_chain` to augment tuplet with proportions ``[1, 2]``.
    Avoid dots::

        >>> staff = Staff("c'8 [ ~ c'16 c'16 ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ~
            c'16
            c'16 ]
        }

    ::

        >>> tie_chain = tietools.get_tie_chain(staff[0])
        >>> tietools.tie_chain_to_tuplet_with_ratio(
        ...     tie_chain, [1, 2], is_diminution=False, dotted=False)
        FixedDurationTuplet(3/16, [c'16, c'8])

    ::

        >>> f(staff)
        \new Staff {
            {
                c'16 [
                c'8
            }
            c'16 ]
        }

    Examle 1c. Change `tie_chain` to augmented tuplet with proportions ``[1, 2, 2]``.
    Avoid dots::

        >>> staff = Staff("c'8 [ ~ c'16 c'16 ]")

    ::

        >>> f(staff)
        \new Staff {
            c'8 [ ~
            c'16
            c'16 ]
        }

    ::

        >>> tie_chain = tietools.get_tie_chain(staff[0])
        >>> tietools.tie_chain_to_tuplet_with_ratio(
        ...     tie_chain, [1, 2, 2], is_diminution=False, dotted=False)
        FixedDurationTuplet(3/16, [c'32, c'16, c'16])

    ::

        >>> f(staff)
        \new Staff {
            \fraction \times 6/5 {
                c'32 [
                c'16
                c'16
            }
            c'16 ]
        }

    Return tuplet.
    '''
    from abjad.tools import tietools

    # check input
    assert isinstance(tie_chain, tietools.TieChain)
    proportions = mathtools.Ratio(proportions)

    # find target duration of fixed-duration tuplet
    target_duration = tie_chain.preprolated_duration

    # find prolated duration of each note in tuplet
    prolated_duration = target_duration / sum(proportions)

    # find written duration of each notes in tuplet
    if is_diminution:
        if dotted:
            basic_written_duration = prolated_duration.equal_or_greater_assignable
        else:
            basic_written_duration = prolated_duration.equal_or_greater_power_of_two
    else:
        if dotted:
            basic_written_duration = prolated_Duration.equal_or_lesser_assignable
        else:
            basic_written_duration = prolated_duration.equal_or_lesser_power_of_two

    # find written duration of each note in tuplet
    written_durations = [x * basic_written_duration for x in proportions]

    # make tuplet notes
    try:
        notes = [notetools.Note(0, x) for x in written_durations]
    except AssignabilityError:
        denominator = target_duration._denominator
        note_durations = [durationtools.Duration(x, denominator) for x in proportions]
        notes = notetools.make_notes(0, note_durations)

    # make tuplet
    tuplet = tuplettools.FixedDurationTuplet(target_duration, notes)

    # replace tie chain with tuplet
    componenttools.move_parentage_and_spanners_from_components_to_components(list(tie_chain), [tuplet])

    # untie tuplet
    spannertools.destroy_spanners_attached_to_component(tuplet, tietools.TieSpanner)

    # return tuplet
    return tuplet



from abjad.tools import componenttools


def get_context_marks_attached_to_any_improper_parent_of_component(component):
    r'''.. versionadded:: 2.0

    Get all context marks attached to any improper parent of `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.ClefMark('treble')(staff)
        ClefMark('treble')(Staff{4})
        >>> contexttools.DynamicMark('f')(staff[0])
        DynamicMark('f')(c'8)

    ::

        >>> f(staff)
        \new Staff {
            \clef "treble"
            c'8 \f
            d'8
            e'8
            f'8
        }

    ::

        >>> contexttools.get_context_marks_attached_to_any_improper_parent_of_component(staff[0])
        (DynamicMark('f')(c'8), ClefMark('treble')(Staff{4}))

    Return tuple.
    '''
    from abjad.tools import contexttools

    result = []

    for component in componenttools.get_improper_parentage_of_component(component):
        for mark in component._marks_for_which_component_functions_as_start_component:
            if isinstance(mark, contexttools.ContextMark):
                result.append(mark)

    return tuple(result)



from abjad.tools import durationtools


def is_bar_line_crossing_leaf(leaf):
    r'''.. versionadded:: 2.0

    True when `leaf` crosses bar line::

        >>> t = Staff("c'8 d'8 e'8 f'8")
        >>> t[2].written_duration *= 2
        >>> contexttools.TimeSignatureMark((2, 8), partial = Duration(1, 8))(t[2])
        TimeSignatureMark((2, 8), partial=Duration(1, 8))(e'4)
        >>> f(t)
        \new Staff {
            c'8
            d'8
            \partial 8
            \time 2/8
            e'4
            f'8
        }
        >>> leaftools.is_bar_line_crossing_leaf(t.leaves[2])
        True

    Otherwise false::

        >>> leaftools.is_bar_line_crossing_leaf(t.leaves[3])
        False

    Return boolean.
    '''
    from abjad.tools import contexttools

    time_signature = contexttools.get_effective_time_signature(leaf)

    if time_signature is None:
        time_signature_duration = durationtools.Duration(4, 4)
    else:
        time_signature_duration = time_signature.duration

    partial = getattr(time_signature, 'partial', durationtools.Duration(0))
    
    shifted_start = (leaf.start_offset - partial) % time_signature_duration

    if time_signature_duration < shifted_start + leaf.prolated_duration:
         return True

    return False



from abjad import *
from abjad.tools import sequencetools
from abjad.tools import rhythmmakertools


def test_OutputIncisedNoteRhythmMaker___call___01():

    prefix_talea, prefix_lengths = [-8], [2]
    suffix_talea, suffix_lengths = [-3], [4]
    talea_denominator = 32
    maker = rhythmmakertools.OutputIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
    measuretools.replace_contents_of_measures_in_expr(staff, leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            r4
            r4
            c'8
        }
        {
            \time 5/8
            c'2 ~
            c'8
        }
        {
            \time 5/8
            c'4
            r16.
            r16.
            r16.
            r16.
        }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 5/8\n\t\tr4\n\t\tr4\n\t\tc'8\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'2 ~\n\t\tc'8\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'4\n\t\tr16.\n\t\tr16.\n\t\tr16.\n\t\tr16.\n\t}\n}"


def test_OutputIncisedNoteRhythmMaker___call___02():

    prefix_talea, prefix_lengths = [-1], [20]
    suffix_talea, suffix_lengths = [-1], [2]
    talea_denominator = 4
    maker = rhythmmakertools.OutputIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
    measuretools.replace_contents_of_measures_in_expr(staff, leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            r4
            r4
            r8
        }
        {
            \time 5/8
            c'2 ~
            c'8
        }
        {
            \time 5/8
            c'8
            r4
            r4
        }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 5/8\n\t\tr4\n\t\tr4\n\t\tr8\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'2 ~\n\t\tc'8\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'8\n\t\tr4\n\t\tr4\n\t}\n}"


def test_OutputIncisedNoteRhythmMaker___call___03():

    prefix_talea, prefix_lengths = [], [0]
    suffix_talea, suffix_lengths = [], [0]
    talea_denominator = 4
    maker = rhythmmakertools.OutputIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
    measuretools.replace_contents_of_measures_in_expr(staff, leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            c'2 ~
            c'8
        }
        {
            \time 5/8
            c'2 ~
            c'8
        }
        {
            \time 5/8
            c'2 ~
            c'8
        }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 5/8\n\t\tc'2 ~\n\t\tc'8\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'2 ~\n\t\tc'8\n\t}\n\t{\n\t\t\\time 5/8\n\t\tc'2 ~\n\t\tc'8\n\t}\n}"


def test_OutputIncisedNoteRhythmMaker___call___04():

    prefix_talea, prefix_lengths = [-1], [1]
    suffix_talea, suffix_lengths = [-1], [1]
    talea_denominator = 8
    prolation_addenda = [1, 0, 3]
    maker = rhythmmakertools.OutputIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator,
        prolation_addenda = prolation_addenda)

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
    measuretools.replace_contents_of_measures_in_expr(staff, leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 4/8
            \times 4/5 {
                r8
                c'2
            }
        }
        {
            \time 4/8
            {
                c'2
            }
        }
        {
            \time 4/8
            \times 4/7 {
                c'2.
                r8
            }
        }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 4/8\n\t\t\\times 4/5 {\n\t\t\tr8\n\t\t\tc'2\n\t\t}\n\t}\n\t{\n\t\t\\time 4/8\n\t\t{\n\t\t\tc'2\n\t\t}\n\t}\n\t{\n\t\t\\time 4/8\n\t\t\\times 4/7 {\n\t\t\tc'2.\n\t\t\tr8\n\t\t}\n\t}\n}"


def test_OutputIncisedNoteRhythmMaker___call___05():

    prefix_talea, prefix_lengths = [-1], [1]
    suffix_talea, suffix_lengths = [-1], [1]
    talea_denominator = 8
    prolation_addenda =    [1, 0, 0, 0, 2]
    secondary_divisions = [3, 1, 4, 1, 3]
    maker = rhythmmakertools.OutputIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator,
        prolation_addenda = prolation_addenda, secondary_divisions = secondary_divisions)

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(measuretools.make_measures_with_full_measure_spacer_skips(divisions))
    measuretools.replace_contents_of_measures_in_expr(staff, leaves)
    measuretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 4/8
            \fraction \times 3/4 {
                r8
                c'4.
            }
            {
                c'8
            }
        }
        {
            \time 4/8
            {
                c'2
            }
        }
        {
            \time 4/8
            {
                c'8
            }
            \fraction \times 3/5 {
                c'2
                r8
            }
        }
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 4/8\n\t\t\\fraction \\times 3/4 {\n\t\t\tr8\n\t\t\tc'4.\n\t\t}\n\t\t{\n\t\t\tc'8\n\t\t}\n\t}\n\t{\n\t\t\\time 4/8\n\t\t{\n\t\t\tc'2\n\t\t}\n\t}\n\t{\n\t\t\\time 4/8\n\t\t{\n\t\t\tc'8\n\t\t}\n\t\t\\fraction \\times 3/5 {\n\t\t\tc'2\n\t\t\tr8\n\t\t}\n\t}\n}"



from abjad.tools.durationtools import Offset
from abjad.tools.timeintervaltools import TimeInterval, TimeIntervalTree


def test_TimeIntervalTree_storage_format_01():

    a = TimeInterval(Offset(-1, 1), Offset(3, 1), {
        'a': 1, 'b': 2, 't': TimeInterval(Offset(2, 1), Offset(3, 1), {'x': 'b'})})
    b = TimeInterval(Offset(0, 1), Offset(1, 1), {'x': 'y'})
    tree = TimeIntervalTree([a, b])

    r'''
    timeintervaltools.TimeIntervalTree([
        timeintervaltools.TimeInterval(
            Offset(-1, 1),
            Offset(3, 1),
            {
                'a': 1,
                'b': 2,
                't': timeintervaltools.TimeInterval(
                    Offset(2, 1),
                    Offset(3, 1),
                    {
                        'x': 'b',
                    }),
            }),
        timeintervaltools.TimeInterval(
            Offset(0, 1),
            Offset(1, 1),
            {
                'x': 'y',
            }),
        ])
    '''

    assert tree.storage_format == "timeintervaltools.TimeIntervalTree([\n\ttimeintervaltools.TimeInterval(\n\t\tOffset(-1, 1),\n\t\tOffset(3, 1),\n\t\t{\n\t\t\t'a': 1,\n\t\t\t'b': 2,\n\t\t\t't': timeintervaltools.TimeInterval(\n\t\t\t\tOffset(2, 1),\n\t\t\t\tOffset(3, 1),\n\t\t\t\t{\n\t\t\t\t\t'x': 'b',\n\t\t\t\t}),\n\t\t}),\n\ttimeintervaltools.TimeInterval(\n\t\tOffset(0, 1),\n\t\tOffset(1, 1),\n\t\t{\n\t\t\t'x': 'y',\n\t\t}),\n\t])"



import copy
from abjad.tools import durationtools
from abjad.tools import marktools
from abjad.tools import notetools
from abjad.tools import tietools


def edit_viola_voice(score, durated_reservoir):

    voice = score['Viola Voice']
    descents = durated_reservoir['Viola']

    for leaf in descents[-1]:
        marktools.Articulation('accent')(leaf)
        marktools.Articulation('tenuto')(leaf)
    copied_descent = copy.deepcopy(descents[-1])
    for leaf in copied_descent:
        if leaf.written_duration == durationtools.Duration(4, 4):
            leaf.written_duration = durationtools.Duration(8, 4)
        else:
            leaf.written_duration = durationtools.Duration(4, 4)
    voice.extend(copied_descent)

    bridge = notetools.Note('e1')
    marktools.Articulation('tenuto')(bridge)
    marktools.Articulation('accent')(bridge)
    voice.append(bridge)

    final_sustain_rhythm = [(6, 4)] * 21 + [(1, 2)]
    final_sustain_notes = notetools.make_notes(['e'], final_sustain_rhythm)
    marktools.Articulation('accent')(final_sustain_notes[0])
    marktools.Articulation('tenuto')(final_sustain_notes[0])
    voice.extend(final_sustain_notes)
    tietools.TieSpanner(final_sustain_notes)
    voice.extend('r4 r2.')




from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class AbjUpScript(DeveloperScript):
    '''Run `ajv svn up -R -C`:

    ::

        bash$ ajv up -h
        usage: abj-up [-h] [--version]

        run `ajv svn up -R -C`

        optional arguments:
          -h, --help  show this help message and exit
          --version   show program's version number and exit

    Return `AbjUpScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'up'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'run `ajv svn up -R -C`'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        from abjad.tools import developerscripttools
        svn_up_script = developerscripttools.SvnUpdateScript()
        svn_up_script(['-R', '-C'])

    def setup_argument_parser(self, parser):
        pass



import math
from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools import tuplettools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class EqualDivisionRhythmMaker(RhythmMaker):
    r'''.. versionadded:: 2.10

    Equal division rhythm-maker::

        >>> maker = rhythmmakertools.EqualDivisionRhythmMaker(4)

    ::

        >>> divisions = [(1, 4), (1, 8), (1, 6), (1, 12)]
        >>> tuplet_lists = maker(divisions)
        >>> tuplets = sequencetools.flatten_sequence(tuplet_lists)

    ::

        >>> staff = Staff(tuplets)

    ::

        >>> f(staff)
        \new Staff {
            {
                c'16
                c'16
                c'16
                c'16
            }
            {
                c'32
                c'32
                c'32
                c'32
            }
            \times 2/3 {
                c'16
                c'16
                c'16
                c'16
            }
            \times 2/3 {
                c'32
                c'32
                c'32
                c'32
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self, leaf_count, is_diminution=True, beam_each_cell=True, beam_cells_together=False):
        assert mathtools.is_integer_equivalent_expr(leaf_count)
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together
            )
        leaf_count = int(leaf_count)
        self._leaf_count = leaf_count
        self._is_diminution = is_diminution

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        result = []
        for division in divisions:
            tuplet = self._make_tuplet(division)
            result.append([tuplet])
        return result

    ### PRIVATE METHODS ###

    def _make_tuplet(self, division):
        numerator, talea_denominator = division
        division_duration = durationtools.Duration(division)
        ratio = self.leaf_count * [1]
        tuplet = tuplettools.make_tuplet_from_duration_and_ratio(
            division_duration, ratio, avoid_dots=True, is_diminution=self.is_diminution)
        return tuplet

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def is_diminution(self):
        '''Return boolean.
        '''
        return self._is_diminution

    @property
    def leaf_count(self):
        '''Leaf count.

        .. note:: add example.

        Return positive integer.
        '''
        return self._leaf_count



from abjad import *


def test_componenttools_get_components_in_expr_with_name_01():

    staff = Staff(r"\new Voice { c'8 d'8 } \new Voice { e'8 f'8 } \new Voice { g'4 }")
    staff[0].name = 'outer voice'
    staff[1].name = 'middle voice'
    staff[2].name = 'outer voice'

    assert componenttools.get_components_in_expr_with_name(staff, 'outer voice') == [staff[0], staff[2]]
    assert componenttools.get_components_in_expr_with_name(staff, 'middle voice') == [staff[1]]



import collections


def all_are_intervals_or_trees_or_empty(input):
    '''Recursively test if all elements of `input` are
    TimeIntervals or TimeIntervalTrees.
    An empty result also return as True.
    '''
    from abjad.tools import timeintervaltools

    def recurse(x):
        if isinstance(x, collections.Iterable) and \
        not isinstance(x, (basestring,
            timeintervaltools.TimeIntervalTree,
            timeintervaltools.TimeInterval)):
            return [a for i in x for a in recurse(i)]
        else:
            return [x]

    return all([isinstance(x, (timeintervaltools.TimeInterval,
        timeintervaltools.TimeIntervalTree))
        for x in recurse(input)])



from abjad import *


def test_markuptools_make_centered_title_markup_01():

    markup = markuptools.make_centered_title_markup('String Quartet')

    assert markup.lilypond_format == '\\markup { \\override #\'(font-name . "Times") \\fontsize #18 \\column { \\center-align { { \\vspace #6 \\line { "String Quartet" } \\vspace #12 } } } }'

    assert markup.indented_lilypond_format == '\\markup {\n\t\\override\n\t\t#\'(font-name . "Times")\n\t\t\\fontsize\n\t\t\t#18\n\t\t\t\\column\n\t\t\t\t{\n\t\t\t\t\t\\center-align\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t{\n\t\t\t\t\t\t\t\t\\vspace\n\t\t\t\t\t\t\t\t\t#6\n\t\t\t\t\t\t\t\t\\line\n\t\t\t\t\t\t\t\t\t{\n\t\t\t\t\t\t\t\t\t\t"String Quartet"\n\t\t\t\t\t\t\t\t\t}\n\t\t\t\t\t\t\t\t\\vspace\n\t\t\t\t\t\t\t\t\t#12\n\t\t\t\t\t\t\t}\n\t\t\t\t\t\t}\n\t\t\t\t}\n\t}'


def test_markuptools_make_centered_title_markup_02():
    '''List of multiple title lines.
    '''

    markup = markuptools.make_centered_title_markup(['String Quartet', 'for the JACK Quartet'])

    assert markup.lilypond_format == '\\markup { \\override #\'(font-name . "Times") \\fontsize #18 \\column { \\center-align { { \\vspace #6 \\line { "String Quartet" } \\line { "for the JACK Quartet" } \\vspace #12 } } } }'

    assert markup.indented_lilypond_format == '\\markup {\n\t\\override\n\t\t#\'(font-name . "Times")\n\t\t\\fontsize\n\t\t\t#18\n\t\t\t\\column\n\t\t\t\t{\n\t\t\t\t\t\\center-align\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\t{\n\t\t\t\t\t\t\t\t\\vspace\n\t\t\t\t\t\t\t\t\t#6\n\t\t\t\t\t\t\t\t\\line\n\t\t\t\t\t\t\t\t\t{\n\t\t\t\t\t\t\t\t\t\t"String Quartet"\n\t\t\t\t\t\t\t\t\t}\n\t\t\t\t\t\t\t\t\\line\n\t\t\t\t\t\t\t\t\t{\n\t\t\t\t\t\t\t\t\t\t"for the JACK Quartet"\n\t\t\t\t\t\t\t\t\t}\n\t\t\t\t\t\t\t\t\\vspace\n\t\t\t\t\t\t\t\t\t#12\n\t\t\t\t\t\t\t}\n\t\t\t\t\t\t}\n\t\t\t\t}\n\t}'



from AbjadBookProcessor import AbjadBookProcessor
from AbjadBookScript import AbjadBookScript
from CodeBlock import CodeBlock
from HTMLOutputFormat import HTMLOutputFormat
from LaTeXOutputFormat import LaTeXOutputFormat
from OutputFormat import OutputFormat
from ReSTOutputFormat import ReSTOutputFormat

_documentation_section = 'internals'



import copy
from abjad.tools import mathtools


def partition_sequence_by_counts(sequence, counts, cyclic=False, overhang=False, copy_elements=False):
    '''.. versionadded:: 1.1

    Example 1a. Partition sequence once by counts without overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(10), [3], cyclic=False, overhang=False)
        [[0, 1, 2]]

    Example 1b. Partition sequence once by counts without overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(16), [4, 3], cyclic=False, overhang=False)
        [[0, 1, 2, 3], [4, 5, 6]]

    Example 2a. Partition sequence cyclically by counts without overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(10), [3], cyclic=True, overhang=False)
        [[0, 1, 2], [3, 4, 5], [6, 7, 8]]

    Example 2b. Partition sequence cyclically by counts without overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(16), [4, 3], cyclic=True, overhang=False)
        [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9, 10], [11, 12, 13]]

    Example 3a. Partition sequence once by counts with overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(10), [3], cyclic=False, overhang=True)
        [[0, 1, 2], [3, 4, 5, 6, 7, 8, 9]]

    Example 3b. Partition sequence once by counts with overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(16), [4, 3], cyclic=False, overhang=True)
        [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9, 10, 11, 12, 13, 14, 15]]

    Example 4a. Partition sequence cyclically by counts with overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(10), [3], cyclic=True, overhang=True)
        [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9]]

    Example 4b. Partition sequence cyclically by counts with overhang::

        >>> sequencetools.partition_sequence_by_counts(
        ...     range(16), [4, 3], cyclic=True, overhang=True)
        [[0, 1, 2, 3], [4, 5, 6], [7, 8, 9, 10], [11, 12, 13], [14, 15]]

    Return list of sequence objects.
    '''
    from abjad.tools import sequencetools

    result = []

    if cyclic:
        if overhang:
            counts = sequencetools.repeat_sequence_to_weight_exactly(counts, len(sequence))
        else:
            counts = sequencetools.repeat_sequence_to_weight_at_most(counts, len(sequence))
    elif overhang:
        weight_counts = mathtools.weight(counts)
        len_sequence = len(sequence)
        if weight_counts < len_sequence:
            counts = list(counts)
            counts.append(len(sequence) - weight_counts)

    for start, stop in mathtools.cumulative_sums_zero_pairwise(counts):
        result.append(type(sequence)(sequence[start:stop]))

    return result



from abjad import *
from abjad.tools.contexttools import TempoMark
from abjad.tools.layouttools import SpacingIndication


def test_SpacingIndication___repr___01():
    '''Repr is evaluable.
    '''

    indication_1 = SpacingIndication(TempoMark(Duration(1, 8), 44), Duration(1, 68))
    indication_2 = SpacingIndication(indication_1)

    assert indication_1 is not indication_2
    assert indication_1 == indication_2



def make_repeated_rests_from_time_signature(time_signature):
    '''.. versionadded:: 2.0

    Make repeated rests from `time_signature`::

        >>> resttools.make_repeated_rests_from_time_signature((5, 32))
        [Rest('r32'), Rest('r32'), Rest('r32'), Rest('r32'), Rest('r32')]

    Return list of newly constructed rests.
    '''
    from abjad.tools import contexttools
    from abjad.tools import resttools

    # afford basic input polymorphism
    time_signature = contexttools.TimeSignatureMark(time_signature)

    # check input
    if time_signature.has_non_power_of_two_denominator:
        raise NotImplementedError(
            'TODO: extend this function for time signatures with non-power-of-two denominators.')

    # make and return repeated rests
    return time_signature.numerator * resttools.Rest((1, time_signature.denominator))



def get_most_distant_sequential_container_in_improper_parentage_of_component(component):
    r'''.. versionadded:: 2.9

    Get first sequential container in the improper parentage of `component`
    such that the parent of sequential container is either a parellel container
    or else none::

        >>> voice_1 = Voice("c'8 d'8")
        >>> voice_2 = Voice("e'8 f'8")
        >>> t = Voice([Container([voice_1, voice_2])])
        >>> t[0].is_parallel = True
        >>> t[0][0].name = 'voice 1'
        >>> t[0][1].name = 'voice 2'

    ::

        >>> f(t)
        \new Voice {
            <<
                \context Voice = "voice 1" {
                    c'8
                    d'8
                }
                \context Voice = "voice 2" {
                    e'8
                    f'8
                }
            >>
        }

    ::

        >>> note = t.leaves[1]

    ::

        >>> componenttools.get_most_distant_sequential_container_in_improper_parentage_of_component(
        ...     note) 
        Voice-"voice 1"{2}

    Return none when no such container exists in the improper parentage of `component`.
    '''
    from abjad.tools import containertools
    from abjad.tools import componenttools

    for component in componenttools.get_improper_parentage_of_component(component):
        if isinstance(component, containertools.Container) and not component.is_parallel:
            if component.parent is None:
                return component
            if isinstance(component.parent, containertools.Container) and \
                component.parent.is_parallel:
                return component



from abjad.tools import durationtools


def fit_metrical_hierarchies_to_expr(expr, metrical_hierarchies,
    discard_final_orphan_downbeat=True, starting_offset=None, denominator=32):
    r'''Find the best-matching sequence of metrical hierarchies for the offsets
    contained in `expr`.

    ::

        >>> metrical_hierarchies = timesignaturetools.MetricalHierarchyInventory(
        ...     [(3, 4), (4, 4), (5, 4)])

    Example 1. Matching a series of hypothetical 4/4 measures:

    ::

        >>> expr = [(0, 4), (4, 4), (8, 4), (12, 4), (16, 4)]
        >>> for x in timesignaturetools.fit_metrical_hierarchies_to_expr(
        ...     expr, metrical_hierarchies):
        ...     print x.implied_time_signature
        ...
        4/4
        4/4
        4/4
        4/4

    Example 2. Matching a series of hypothetical 5/4 measures:

    ::

        >>> expr = [(0, 4), (3, 4), (5, 4), (10, 4), (15, 4), (20, 4)]
        >>> for x in timesignaturetools.fit_metrical_hierarchies_to_expr(
        ...     expr, metrical_hierarchies):
        ...     print x.implied_time_signature
        ...
        5/4
        5/4
        5/4
        5/4

    Offsets are coerced from `expr` via `durationtools.count_offsets_in_expr()`.

    MetricalHierarchies are coerced from `metrical_hierarchies` via `MetricalHierarchyInventory`.

    Return list.
    '''
    from abjad.tools import timesignaturetools

    offset_counter = durationtools.count_offsets_in_expr(expr)
    ordered_offsets = sorted(offset_counter, reverse=True)
    if not ordered_offsets:
        return []

    if starting_offset is None:
        start_offset = durationtools.Offset(0)
    else:
        start_offset = durationtools.Offset(starting_offset)

    metrical_hierarchy_inventory = timesignaturetools.MetricalHierarchyInventory(metrical_hierarchies)
    longest_hierarchy = sorted(metrical_hierarchy_inventory, key=lambda x: x.duration, reverse=True)[0]
    longest_kernel_duration = max(x.duration for x in metrical_hierarchy_inventory)
    kernels = [x.generate_offset_kernel_to_denominator(denominator) for x in metrical_hierarchy_inventory]
    
    current_start_offset = start_offset
    selected_hierarchies = []

    while len(ordered_offsets) and ordered_offsets[-1] < current_start_offset:
        ordered_offsets.pop()

    while len(ordered_offsets) and current_start_offset <= ordered_offsets[-1]:
        if len(ordered_offsets) == 1:
            if discard_final_orphan_downbeat:
                if ordered_offsets[0] == current_start_offset:
                    break
        current_stop_offset = current_start_offset + longest_kernel_duration
        current_offset_counter = {}
        for offset in reversed(ordered_offsets):
            if current_start_offset <= offset <= current_stop_offset:
                current_offset_counter[offset - current_start_offset] = offset_counter[offset]
            else:
                break
        if not current_offset_counter:
            winner = longest_hierarchy
        else: 
            candidates = []
            for i, kernel in enumerate(kernels):
                response = kernel(current_offset_counter)
                candidates.append((response, i))
            candidates.sort(key=lambda x: x[0], reverse=True)
            response, index = candidates[0]
            winner = metrical_hierarchy_inventory[index]
        selected_hierarchies.append(winner)
        current_start_offset += winner.duration 
        while len(ordered_offsets) and ordered_offsets[-1] < current_start_offset:
            ordered_offsets.pop()

    return selected_hierarchies



from abjad import *


def test_PaperBlock_01():

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.top_margin = 15
    paper_block.left_margin = 15
    paper_block.markup_system_spacing__basic_distance = 8

    r'''
    \paper {
        left-margin = #15
        markup-system-spacing #'basic-distance = #8
        top-margin = #15
    }
    '''

    assert paper_block.lilypond_format == "\\paper {\n\tleft-margin = #15\n\tmarkup-system-spacing #'basic-distance = #8\n\ttop-margin = #15\n}"


def test_PaperBlock_02():
    '''Make sure scheme vectors format correctly.
    '''

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.system_system_spacing = layouttools.make_spacing_vector(0, 0, 12, 0)

    r'''
    \paper {
        system-system-spacing = #'((basic_distance . 0) (minimum_distance . 0) (padding . 12) (stretchability . 0))
    }
    '''

    assert paper_block.lilypond_format == "\\paper {\n\tsystem-system-spacing = #'((basic_distance . 0) (minimum_distance . 0) (padding . 12) (stretchability . 0))\n}"


def test_PaperBlock_03():

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.append(schemetools.Scheme([
        'define', 'fonts', [
            'make-pango-font-tree',
            schemetools.Scheme('Baskerville', force_quotes=True),
            schemetools.Scheme('Baskerville', force_quotes=True),
            schemetools.Scheme('Baskerville', force_quotes=True),
            schemetools.Scheme(['/', 14, 20])
            ]
        ]))

    r'''
    \paper {
        #(define fonts (make-pango-font-tree "Baskerville" "Baskerville" "Baskerville" (/ 14 20)))
    }
    '''

    assert paper_block.lilypond_format == '\\paper {\n\t#(define fonts (make-pango-font-tree "Baskerville" "Baskerville" "Baskerville" (/ 14 20)))\n}'

